/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * adc.h -- A/D interface header file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#ifndef _ADC_H_
#define _ADC_H_

#define ADC_CLKSRC_SOFTWARE 0
#define ADC_CLKSRC_INTERNAL 1
#define ADC_CLKSRC_EXTERNAL 2

#define ADC_TRIGSRC_INTERNAL 0
#define ADC_TRIGSRC_REFERENCE 1
#define ADC_TRIGSRC_EXTERNAL 2

#define ADC_TRIGMDE_POS 0
#define ADC_TRIGMDE_NEG 1
#define ADC_TRIGMDE_RISE 2
#define ADC_TRIGMDE_FALL 3

#define ADC_IRQ_DISABLED 0
#define ADC_IRQ_ENABLED 1

#define ADC_RANGE_UNISE 0
#define ADC_RANGE_BIPSE 1
#define ADC_RANGE_UNIDIF 2
#define ADC_RANGE_BIPDIF 3

#define ADC_STATUS_IDLE 0
#define ADC_STATUS_BUSY 1

#define ADC_BUFFER_SIZE 1048576
#define ADC_FREQUENCY_MAX 400000
#define ADC_FIFO_SIZE 4096
#define ADC_FIFO_HALF 2048
#define ADC_CHANLST_MAX 64
#define ADC_CLK_TIMEOUT 20
#define ADC_DBLBUF_SIZE 4096
#define ADC_NUM_CHAN64 64

#define ADC_DATA_MODE_RAW 0
#define ADC_DATA_MODE_VOLT 1

struct ADCBuffer_t {
	unsigned short *data;
	unsigned long size;
	unsigned long samples;
	unsigned long head;
	unsigned long tail;
	unsigned long wrap;
	unsigned char overrun;
	spinlock_t lock;
};
#define pADCBuffer_t struct ADCBuffer_t*

struct ADCChannel_t {
	unsigned char channel;
	unsigned char gain;
	unsigned char range;
	unsigned char reference;
};
#define pADCChannel_t struct ADCChannel_t*

struct ADCREG_t {
	unsigned char	softclk;
	unsigned char	eolints;
	unsigned char	trigmode;
	unsigned char trigsrc;
	unsigned char status;
	unsigned char clksrc;
	unsigned short lastval;
};
#define pADCREG_t struct ADCREG_t*

struct ADC_t {
	spinlock_t lock;
	void *datafifo;
	void *chanfifo;
	void *control;
	void *status;
	struct ADCBuffer_t buffer;
	struct ADCChannel_t chanlst[ADC_CHANLST_MAX];
	unsigned char assigned[ADC_CHANLST_MAX];
	struct ADCREG_t reg;
	pCNT_t pcnt;
	unsigned char channels;
	unsigned long frequency;
	unsigned char clksrc;
	unsigned char trigsrc;
	unsigned char clkmode;
	unsigned char testmux;
	unsigned char test;
	unsigned char gain;
	unsigned char range;
	unsigned char enable;
	unsigned char resetfifo;
	unsigned char fifomode;
	unsigned short chanlstln;
	unsigned short chanidx;
	unsigned char fifostatus;
	unsigned char ints_enable;
	unsigned char ADconvert;
	unsigned char mode;
	unsigned short waterlevel;
	unsigned char datamode;
};
#define pADC_t struct ADC_t*

void adc_init( pADC_t padc, pCNT_t pcnt, void *datafifo, void *chanfifo,
	       void *control, void *status );
void adc_uninit( pADC_t padc );
void adc_ctrlupdate( pADC_t padc );
void adc_fullstop( pADC_t padc );
long adc_isbusy( pADC_t padc );
void adc_start( pADC_t padc );
void adc_stop( pADC_t padc );
long adc_setclock( pADC_t padc, long clksrc, long trigsrc );
void adc_fiforst( pADC_t padc );
long adc_fifoempty( pADC_t padc );
void adc_softclock( pADC_t padc );
void adc_progchanlist( pADC_t padc );
void adc_buffrst( pADCBuffer_t pbuf );
void adc_reset( pADC_t padc );
unsigned long adc_setfrequency( pADC_t padc, unsigned long value );
long adc_getsinglevalue( pADC_t padc, long* buffer );
long adc_bintovolt( unsigned short bin, int bipolar, int gain );
void adc_irqenable( pADC_t padc );
void adc_irqdisable( pADC_t padc );
long adc_settrigmode( pADC_t padc, long mode );
long adc_isdiffchan( unsigned char chan );
unsigned char adc_getdiffref( unsigned char chan );
long adc_isdifferential( long range );
long adc_isbipolar( long range );
long adc_setrange( pADC_t padc, unsigned char range );
long adc_setdatamode( pADC_t padc, unsigned char mode );
long adc_getwaterlevel( pADC_t padc );
long adc_dmatobuf( pADCBuffer_t pbuf, unsigned long* dmabuf,
		   unsigned short waterlevel );
unsigned long adc_getdata( pADC_t padc, long *buffer,
			   unsigned long samples );
unsigned long adc_getdataraw( pADCBuffer_t pbuf, unsigned short *buffer,
			      unsigned long samples);
unsigned long adc_getunread( pADCBuffer_t pbuf );
void encodechannel( unsigned char chan, unsigned char range, unsigned char gain,
		    unsigned char *mux, unsigned char *group, unsigned char *pairs );
long adc_fifotest( pADC_t padc );

#endif /*_ADC_H_*/
/**
 * End of File
 */
