/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * caldac.c -- Calibration D/A interface source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#include "stdinc.h"
#include "caldac.h"

void caldac_init( pCALDAC_t pcaldac, void *control )
{
	spin_lock_init(&pcaldac->lock);
	spin_lock(&pcaldac->lock);
	pcaldac->control = control;
	spin_unlock(&pcaldac->lock);
}

long caldac_write( pCALDAC_t pcaldac, unsigned char position,
		   unsigned char value )
{
	long i, bm;
	value &= 0xff;

	if(position > 7)
		return -EINVAL;

	bm=0x8;
	for(i=0; i<4; i++) {
		caldac_writebit(pcaldac, bm & (position + 1));
		bm>>=1;
	}

	bm=0x80;
	for(i=0;i<8;i++) {
		caldac_writebit(pcaldac, bm & value);
		bm >>= 1;
	}
	caldac_loaddata(pcaldac);
	return 0;
}

void caldac_writebit( pCALDAC_t pcaldac, unsigned char value )
{
	unsigned long cntrl;
	cntrl = readl(pcaldac->control) & CALDAC_LOW;
	if(value > 1)
		value = 1;
	writel(cntrl | CALDAC_PR, pcaldac->control);
	udelay(CALDAC_DELAY);
	writel(cntrl | (value * CALDAC_SDI) | CALDAC_PR, pcaldac->control);
	udelay(CALDAC_DELAY);
	writel(cntrl | (value * CALDAC_SDI) | CALDAC_CLK | CALDAC_PR,
	       pcaldac->control);
	udelay(CALDAC_DELAY);
	writel(cntrl | (value * CALDAC_SDI) | CALDAC_PR, pcaldac->control);
	udelay(CALDAC_DELAY);
	writel(cntrl | CALDAC_PR, pcaldac->control);
	udelay(CALDAC_DELAY);
}

void caldac_enable( pCALDAC_t pcaldac )
{
	unsigned long cntrl;
	cntrl = readl(pcaldac->control) & CALDAC_LOW;
	writel(cntrl, pcaldac->control);
}

void caldac_disable( pCALDAC_t pcaldac )
{
	unsigned long cntrl;
	cntrl = readl(pcaldac->control) & CALDAC_LOW;
	writel(cntrl | CALDAC_PR, pcaldac->control);
}

void caldac_loaddata( pCALDAC_t pcaldac )
{
	unsigned long cntrl;
	cntrl = readl(pcaldac->control) & CALDAC_LOW;
	writel(cntrl | CALDAC_LD | CALDAC_PR, pcaldac->control);
	udelay(CALDAC_DELAY);
	writel(cntrl | CALDAC_LD | CALDAC_CLK | CALDAC_PR, pcaldac->control);
	udelay(CALDAC_DELAY);
	writel(cntrl | CALDAC_LD | CALDAC_PR, pcaldac->control);
	udelay(CALDAC_DELAY);
	writel(cntrl | CALDAC_PR, pcaldac->control);
	udelay(CALDAC_DELAY);
}

/**
 * End of File
 */
