/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * counter.c -- counter-timer source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#include "stdinc.h"
#include "counter.h"

void cnt_write( pCNT_t pcnt, unsigned long val )
{
	spin_lock(&pcnt->lock);
	pcnt->value = val;
	writel(val, pcnt->data);
	spin_unlock(&pcnt->lock);
}

unsigned long cnt_read( pCNT_t pcnt )
{
	unsigned long value;
	spin_lock(&pcnt->lock);
	value = pcnt->value;
	spin_unlock(&pcnt->lock);
	return value;
}

void cnt_update( pCNT_t pcnt )
{
	pcnt->lastcfg = (pcnt->status) |
			(pcnt->clksrc<<1) |
			(pcnt->mode<<3) |
			(pcnt->irq<<4);
			writel(pcnt->lastcfg, pcnt->control);
}

long cnt_configure( pCNT_t pcnt, unsigned char clocksource, unsigned char mode,
		    unsigned char irq )
{
	if(clocksource > CNT_SRC_EXT)
		clocksource = CNT_SRC_EXT;
	if(mode > CNT_MODE_TOGGLE)
		mode = CNT_MODE_TOGGLE;
	if(irq > CNT_IRQ_ENABLED)
		irq = CNT_IRQ_ENABLED;

	spin_lock(&pcnt->lock);
	if(clocksource == 2)
		clocksource++;
	pcnt->clksrc = clocksource;
	pcnt->mode = mode;
	pcnt->irq = irq;
	cnt_update(pcnt);
	spin_unlock(&pcnt->lock);
	return 0;
}

long cnt_isbusy( pCNT_t pcnt )
{
	unsigned char status;
	spin_lock(&pcnt->lock);
	status = pcnt->status;
	spin_lock(&pcnt->lock);
	return status;
}

void cnt_enable( pCNT_t pcnt )
{
	spin_lock(&pcnt->lock);
	pcnt->status = CNT_STATUS_BUSY;
	cnt_update(pcnt);
	spin_unlock(&pcnt->lock);
}

void cnt_disable( pCNT_t pcnt )
{
	spin_lock(&pcnt->lock);
	pcnt->status = CNT_STATUS_IDLE;
	cnt_update(pcnt);
	spin_unlock(&pcnt->lock);
}

void cnt_init( pCNT_t pcnt, unsigned char index, void *control, void *data )
{
	spin_lock_init(&pcnt->lock);
	spin_lock(&pcnt->lock);
	pcnt->control = control;
	pcnt->data = data;
	pcnt->index = index;
	pcnt->clksrc = CNT_SRC_20MHZ;
	pcnt->status = CNT_STATUS_IDLE;
	pcnt->mode = CNT_MODE_PULSE;
	pcnt->irq = CNT_IRQ_DISABLED;
	pcnt->count = 0;
	cnt_update(pcnt);
	spin_unlock(&pcnt->lock);
}

/**
 * End of File
 */
