/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * dac.h -- D/A interface header file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#ifndef _DAC_H_
#define _DAC_H_

#define DAC_MODE_DISABLED 0
#define DAC_MODE_PIO 1
#define DAC_MODE_ENABLED 2
#define DAC_MODE_PATTERN 3
#define DAC_STATUS_IDLE 0
#define DAC_STATUS_BUSY 1
#define DAC_FIFO_SIZE 4096

struct DACREG_t
{
	unsigned char fifohalf;
	unsigned char fifoempty;
	unsigned char mode;
	unsigned short lastvalue;
};
#define pDACREG_t struct DACREG_t*

struct DAC_t
{
	spinlock_t lock;
	void *ctrl;
	void *data;
	unsigned short *buffer;
	unsigned long buffersize;
	unsigned char index;
	unsigned char status;
	unsigned char runmode;
	struct DACREG_t reg;
	pCNT_t pcnt;
};
#define pDAC_t struct DAC_t*

void dac_init( pDAC_t pdac, void *control, void *datafifo, pCNT_t pcounter );
void dac_uninit( pDAC_t pdac );
long dac_isbusy( pDAC_t pdac );
unsigned short dac_volttobin( long uv );
void dac_update( pDAC_t pdac );
void dac_pause( pDAC_t pdac );
void dac_progio( pDAC_t pdac, long uv );
void dac_reset( pDAC_t pdac );
long dac_adddata( pDAC_t pdac, void *buffer, unsigned long length );
void dac_updatefifo( pDAC_t pdac );
void dac_enable( pDAC_t pdac );
void dac_disable( pDAC_t pdac );
void dac_pattern( pDAC_t pdac );
void dac_writefifo( pDAC_t pdac, unsigned short bin );
long dac_configure( pDAC_t pdac, long freq, long clksrc, long gatesrc );
void dac_continue( pDAC_t pdac );
void dac_writebin( pDAC_t pdac, unsigned short bin );
long dac_isconfigured( pDAC_t pdac );
void dac_setmode( pDAC_t pdac, unsigned char mode );

#endif /*_DAC_H_*/
/**
 * End of File
 */
