/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * eeprom.c -- Eeprom interface source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#include "stdinc.h"
#include "eeprom.h"

void ee_init( pEeprom_t pee, void *control )
{
	pee->control = control;
}

void ee_writebit( pEeprom_t pee, unsigned char value )
{
	if(value > 0)
		value = 1;
	pee->_di = value;
	ee_clklo(pee);
	ee_clkhi(pee);
	ee_clklo(pee);
}

unsigned char ee_readbit( pEeprom_t pee )
{
	unsigned char value;
	ee_clklo(pee);
	ee_clkhi(pee);
	value = pee->_do;
	ee_clklo(pee);
	return value;
}

void ee_disable( pEeprom_t pee )
{
	pee->cs = 0;
	ee_update(pee);
}

void ee_writeenable( pEeprom_t pee )
{
	ee_disable(pee);
	ee_enable(pee);
	ee_writebit(pee, 1);
	ee_writebit(pee, 0);
	ee_writebit(pee, 0);
	ee_writebit(pee, 1);
	ee_writebit(pee, 1);
	ee_writebit(pee, 0);
	ee_writebit(pee, 0);
	ee_writebit(pee, 0);
	ee_writebit(pee, 0);
	ee_disable(pee);
}

long ee_read( pEeprom_t pee, unsigned short position )
{
	unsigned long val, i, bm;
	val = 2;
	if(position >= EE_SIZE)
		return -EINVAL;
	ee_disable(pee);
	ee_enable(pee);
	ee_writebit(pee, 1);
	ee_writebit(pee, 1);
	ee_writebit(pee, 0);
	bm = 0x20;
	for(i=0; i<6; i++) {
		ee_writebit(pee, position&bm);
		bm >>= 1;
	}
	val = 0;
	bm = 0x8000;
	for(i=0; i<16; i++) {
		if(ee_readbit(pee)>0)
			val |= bm;
			bm >>= 1;
	}
	ee_disable(pee);
	return (long)val;
}

long ee_write( pEeprom_t pee, unsigned short position, unsigned short value )
{
	unsigned int bm, i;
	if(position >= EE_SIZE)
		return -EINVAL;
	ee_writeenable(pee);
	ee_enable(pee);
	ee_writebit(pee, 1);
	ee_writebit(pee, 0);
	ee_writebit(pee, 1);
	bm = 0x20;
	for(i=0; i<6; i++) {
		ee_writebit(pee, position&bm);
		bm >>= 1;
	}
	bm = 0x8000;
	for(i=0; i<16; i++)
	{
		ee_writebit(pee, value&bm);
		bm >>= 1;
	}
	ee_disable(pee);
	ee_clk(pee);
	ee_enable(pee);
	ee_clk(pee);
	ee_waitforend(pee);
	ee_disable(pee);
	ee_clk(pee);
	ee_clk(pee);
	return 0;
}

void ee_waitforend( pEeprom_t pee )
{
	do {
		ee_clk(pee);
		ee_update(pee);
		udelay(100);
	} while(!(pee->_do));
}

void ee_clklo( pEeprom_t pee )
{
	pee->clk = 0;
	ee_update(pee);
}

void ee_clkhi( pEeprom_t pee )
{
	pee->clk = 1;
	ee_update(pee);
}

void ee_enable( pEeprom_t pee )
{
	pee->cs = 1;
	ee_update(pee);
}

void ee_clk( pEeprom_t pee )
{
	ee_clklo(pee);
	ee_clkhi(pee);
	ee_clklo(pee);
}

void ee_update( pEeprom_t pee )
{
	unsigned long val;
	val = (pee->cs * EE_CS + pee->clk * EE_CLK + pee->_di * EE_DI) & 0x000f0000;
	writel(val, pee->control);
	udelay(10);
	if((readl(pee->control) & EE_DO) > 0)
		pee->_do = 1;
	else
		pee->_do = 0;
}

/**
 * End of File
 */
