/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * ioctl.c -- I/O interface source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#include "stdinc.h"
#include "pci703def.h"
#include "caldac.h"
#include "eeprom.h"
#include "counter.h"
#include "adc.h"
#include "dac.h"
#include "isr.h"
#include "pci703.h"
#include "version.h"
#include "ioctl.h"

int ioctl_query( pBoardDevice_t pdev, unsigned long arg )
{
#define NUMADCRANGES 9
#define NUMADCMODES 4
#define NUMADCCLKS 3
#define NUMADCTRIGS 3
#define NUMADCTRIGMODES 4

	unsigned long param[2];
	const long ADCRANGES[NUMADCRANGES] = {
		20000000,10000000,5000000,2000000,1000000,500000,200000 ,100000,50000 };
	const long ADCMODES[NUMADCMODES] = { UNI_SINGLE,BI_SINGLE,UNI_DIFF,BI_DIFF };
	const long ADCCLKS[NUMADCCLKS] = {
		AD_CLK_UNDEFINED,AD_CLK_INTERNAL,AD_CLK_EXTERNAL };
	const long ADCTRIGS[NUMADCTRIGS] = {
		AD_TRIG_INTERNAL,AD_TRIG_EXTERNAL,AD_TRIG_REF };
	const long ADCTRIGMODES[NUMADCTRIGMODES] = {
		AD_TRIG_POS,AD_TRIG_NEG,AD_TRIG_RIS,AD_TRIG_FAL };

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	switch(param[0]) {
	case BRDSERIALNO:
		param[0] = pdev->devinfo.serialnumber;
		break;
	case BRDREV:
		param[0] = pdev->devinfo.revision;
		break;
	case BRDTYPE:
		param[0] = pdev->devinfo.boardtype;
		break;
	case BRDYEAR:
		param[0] = pdev->devinfo.year;
		break;
	case BRDMONTH:
		param[0] = pdev->devinfo.month;
		break;
	case BRDDAY:
		param[0] = pdev->devinfo.day;
		break;
	case BRDBUSTYPE:
		param[0] = PC_BUS_PCI;
		break;
	case DRVMAJOR:
		param[0] = (PCI703DRV_VERSION>>16)&0xff;
		break;
	case DRVMINOR:
		param[0] = (PCI703DRV_VERSION>>8)&0xff;
		break;
	case DRVBUILD:
		param[0] = PCI703DRV_VERSION&0xff;
		break;
	case ADNUMCHAN:
		param[0] = pdev->devinfo.adcchan;
		break;
	case ADUNREAD:
		param[0] = adc_getunread(&pdev->adc.buffer);
		break;
	case ADMAXFREQ:
		param[0] = 400000;
		break;
	case ADFIFOSIZE:
		param[0] = 4096;
		break;
	case ADBUFFALLOC:
		if(pdev->adc.buffer.data != NULL)
			param[0]=1;
		else
			param[0]=0;
		break;
	case ADCLRBUF:
		adc_buffrst(&pdev->adc.buffer);
		param[0] = 0;
		break;
	case ADIRQLEVEL:
		pdev->adc.waterlevel = param[1];
		param[0] = 0;
		break;
	case ADNUMRANGES:
		param[0] = NUMADCRANGES;
		break;
	case ADRANGES:
		if(param[1] >= NUMADCRANGES)
			param[0] = 0;
		else
			param[0] = ADCRANGES[param[1]];
		break;
	case ADNUMMODES:
		param[0] = NUMADCMODES;
		break;
	case ADMODES:
		if(param[1] >= NUMADCMODES)
			param[0] = 0;
		else
			param[0] = ADCMODES[param[1]];
		break;
	case ADNUMGAINS:
		param[0] = 0;
		break;
	case ADGAINS:
		param[0] = 0;
		break;
	case ADNUMTRIG:
		param[0] = NUMADCTRIGS;
		break;
	case ADTRIG:
		if(param[1] >= NUMADCTRIGS)
			param[0] = 0;
		else
			param[0] = ADCTRIGS[param[1]];
		break;
	case ADNUMTRIGMODE:
		param[0] = NUMADCTRIGMODES;
		break;
	case ADTRIGMODE:
		if(param[1] >= NUMADCTRIGMODES)
			param[0] = 0;
		else
			param[0] = ADCTRIGMODES[param[1]];
		break;
	case ADNUMCLK:
		param[0] = NUMADCCLKS;
		break;
	case ADCLK:
		if(param[1] >= NUMADCCLKS)
			param[0] = 0;
		else
			param[0] = ADCCLKS[param[1]];
		break;
	case ADSETDATAMODE:
		param[0] = adc_setdatamode(&pdev->adc, param[1]);
		break;
	case ADREFGNDPIN:
		param[0] = 1;
		break;
	case ADNUMGATES:
		param[0] = 0;
		break;
	case ADGATE:
		param[0] = 0;
		break;
	case ADDMATEST:
		adc_fiforst(&pdev->adc);
		param[0] = dma_test(&pdev->irq);
		break;
	case DANUMCHAN:
		param[0] = pdev->devinfo.dacchan;
		break;
	case CTNUM:
		param[0] = pdev->devinfo.cntchan;
		break;
	case DIONUMPORT:
		param[0] = pdev->devinfo.diochan;
		break;
	case DIOQRYPORT:
		param[0] = DIOINOROUT;
		break;
	case DIOPORTWIDTH:
		param[0] = 1;
		break;
	case INTNUMSRC:
		param[0] = 0;
		break;
	default:
		param[0] = 0;
	}
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return 0;
}

int ioctl_diowrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2], port, value;

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	value = param[1] & 0x01;

	if(port > 17)
		return -EINVAL;
	if(value > 1)
		return -EINVAL;

	if(port <= 7) {
		pdev->dio[0].setup &= (0xff - (1<<port));
		pdev->dio[0].setup |= (1<<port);
		writel(pdev->dio[0].setup, pdev->ptrmem1 + PCI703_REG_DIO_CONFIG);
		pdev->dio[0].value &= (0xff - (1<<port));
		pdev->dio[0].value |= (value<<port);
		writel(pdev->dio[0].value, pdev->ptrmem1 + PCI703_REG_DIO_DATA);
	}

	if((port > 7) && (port <= 17)) {
		port -= 8;
		pdev->dio[1].setup &= (0xff - (1<<port));
		pdev->dio[1].setup |= (1<<port);
		writel(pdev->dio[1].setup, pdev->ptrmem1 + PCI703_REG_PIO_CONFIG);
		pdev->dio[1].value &= (0xff - (1<<port));
		pdev->dio[1].value |= (value<<port);
		writel(pdev->dio[1].value, pdev->ptrmem1 + PCI703_REG_PIO_DATA);
	}
	return 0;
}

int ioctl_dioread( pBoardDevice_t pdev, unsigned long arg)
{
	unsigned long param[2], port, value;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port = param[0];
	value = 0;
	if(port > 17)
		return -EINVAL;

	if(port <= 7) {
		pdev->dio[0].setup &= (0xff - (1<<port));
		writel(pdev->dio[0].setup, pdev->ptrmem1 + PCI703_REG_DIO_CONFIG);
		value = readl(pdev->ptrmem1 + PCI703_REG_DIO_DATA);
		value =( value>>port) & 0x01;
	}
	if((port > 7) && (port <= 17)) {
		port -= 8;
		pdev->dio[1].setup &= (0xff - (1<<port));
		writel(pdev->dio[1].setup, pdev->ptrmem1 + PCI703_REG_PIO_CONFIG);
		value = readl(pdev->ptrmem1 + PCI703_REG_PIO_DATA);
		value = (value>>port) & 0x01;
	}
	param[0] = value;
	copy_to_user((void *)arg, param,sizeof(unsigned long));
	return 0;
}

int ioctl_miocfg( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2], port, value;

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port=param[0];
	value=param[1];
	param[0]=0;
	copy_to_user((void *)arg,param,sizeof(unsigned long));
	return -EINVAL;
}

int ioctl_ctwrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2], port, value;

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	value = param[1];
	if(port >= pdev->devinfo.cntchan)
		return -EINVAL;
	cnt_write(&pdev->cnt[port], value);
	return 0;
}

int ioctl_ctread( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[1], port;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port = param[0];
	if(port >= pdev->devinfo.cntchan)
		return -EINVAL;
	param[0] = cnt_read(&pdev->cnt[port]);
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return 0;
}

int ioctl_ctcfg( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[5], port, mode, type, clksrc;

	copy_from_user(param, (void *)arg, 4*sizeof(unsigned long));
	port = param[0];
	mode = param[1];
	type = param[2];
	clksrc = param[3];
	if(port >= pdev->devinfo.cntchan)
		return -EINVAL;
	return cnt_configure(&pdev->cnt[port], clksrc, mode, type);
}

int ioctl_ctsoftgate( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2], port, mode;

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	mode = param[1];
	if(port >= pdev->devinfo.cntchan)
		return -EINVAL;
	if(mode > 0)
		cnt_enable(&pdev->cnt[port]);
	else
		cnt_disable(&pdev->cnt[port]);
	return 0;
}

int ioctl_dawrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2], port;
	long value;

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	value = param[1];
	if(port >= pdev->devinfo.dacchan)
		return -EINVAL;
	if((value > 10000000) || (value < -10000000))
		return -EINVAL;
	if(dac_isbusy(&pdev->dac[port]))
		return -EBUSY;
	dac_progio(&pdev->dac[port], value);
	return 0;
}

int ioctl_dacontrol( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long chan;
	unsigned long command;
	unsigned long param[2];

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	chan = param[0];
	command = param[1];
	if(chan >= pdev->devinfo.dacchan)
		return -EINVAL;

	switch (command) {
	case DA_COMMAND_NULL:
		break;
	case DA_COMMAND_START:
		dac_enable(&pdev->dac[chan]);
		break;
	case DA_COMMAND_STOP:
		dac_disable(&pdev->dac[chan]);
		break;
	case DA_COMMAND_PAUSE:
		dac_pause(&pdev->dac[chan]);
		break;
	case DA_COMMAND_CONT:
		dac_continue(&pdev->dac[chan]);
		break;
	default:
		return -EINVAL;
	}
	return 0;
}

int ioctl_dablock( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[4];
	unsigned long chan;
	unsigned long length;
	unsigned long cont;

	copy_from_user(param, (void *)arg, 3*sizeof(unsigned long));
	chan = param[0];
	length = param[1];
	cont = param[2];
	if(chan >= pdev->devinfo.dacchan)
		return -EINVAL;
	if((cont != 0) && (cont != 1))
		return -EINVAL;
	if(dac_isbusy(&pdev->dac[chan]))
		return -EBUSY;
	dac_setmode(&pdev->dac[chan], cont + 2);
	return dac_adddata(&pdev->dac[chan], (void *)(arg + 3), length);
}

int ioctl_dacfg( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[4];
	unsigned long chan, freq, clksrc, gatesrc;

	copy_from_user(param, (void *)arg, 4*sizeof(unsigned long));
	chan = param[0];
	freq = param[1];
	clksrc = param[2];
	gatesrc = param[3];
	if(chan >= pdev->devinfo.dacchan)
		return -EINVAL;
	if(dac_isbusy(&pdev->dac[chan]))
		return -EBUSY;
	dac_reset(&pdev->dac[chan]);
	return dac_configure(&pdev->dac[chan], freq, clksrc, gatesrc);
}

int ioctl_adsingle( pBoardDevice_t pdev, unsigned long arg )
{
	pADC_t padc = &pdev->adc;
	unsigned long param[2];
	unsigned char chan, gain;
	long uv;

	if(adc_isbusy(padc))
		return -EBUSY;
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	chan = param[0];
	gain = param[1];
	if(gain > 7) gain = 7;

	spin_lock(&padc->lock);
	adc_reset(padc);
	padc->clksrc = ADC_CLKSRC_SOFTWARE;
	padc->trigsrc = ADC_TRIGSRC_INTERNAL;
	padc->chanlstln = 1;
	padc->chanlst[0].channel = chan;
	padc->chanlst[0].range = padc->range;
	padc->chanlst[0].gain = gain;
	padc->chanlst[0].reference = 1;
	adc_progchanlist(padc);
	spin_unlock(&padc->lock);

	adc_start(padc);
	adc_softclock(padc);
	adc_softclock(padc);
	adc_stop(padc);

	if(adc_fifoempty(padc))
		return -EIO;
	if(adc_getsinglevalue(padc, &uv))
		return -EIO;
	copy_to_user((void *)arg, &uv, sizeof(long));
	return 0;
}

int ioctl_adconfig( pBoardDevice_t pdev, unsigned long arg )
{
	pADC_t padc =& pdev->adc;
	unsigned long param[2];

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	if(adc_isbusy(&pdev->adc))
		return -EBUSY;

	spin_lock(&padc->lock);
	padc->range = (unsigned char)param[0];
	if(padc->range > 3)
		padc->range = 3;
	spin_unlock(&padc->lock);
	adc_reset(&pdev->adc);
	irq_reset(&pdev->irq);
	return 0;
}

int ioctl_adstop( pBoardDevice_t pdev, unsigned long arg )
{
	adc_stop(&pdev->adc);
	return 0;
}

int ioctl_adstart(pBoardDevice_t pdev, unsigned long arg)
{
	if(adc_isbusy(&pdev->adc))
		return -EBUSY;
	adc_start(&pdev->adc);
	return 0;
}

int ioctl_adclksrc( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[1];

	pADC_t padc = &pdev->adc;
	if(adc_isbusy(padc))
		return -EBUSY;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	spin_lock(&padc->lock);
	padc->clksrc = param[0] & 0xff;
	padc->trigsrc = (padc->clksrc>>8) & 0x0f;
	if(padc->clksrc == 0)
		padc->clksrc=ADC_CLKSRC_INTERNAL;
	if(padc->clksrc > ADC_CLKSRC_EXTERNAL)
		padc->clksrc=ADC_CLKSRC_EXTERNAL;
	spin_unlock(&padc->lock);
	return 0;
}

int ioctl_adfreq( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[1];

	if(adc_isbusy(&pdev->adc))
		return -EBUSY;
	copy_from_user(param,(void *)arg,sizeof(unsigned long));

	param[0] = adc_setfrequency(&pdev->adc,param[0]);
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return 0;
}

int ioctl_adchlst( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[129], chlstsz, pos;
	unsigned long dm;
	pADCChannel_t pchan;
	pADC_t padc = &pdev->adc;

	if(adc_isbusy(&pdev->adc))
		return -EBUSY;
	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	chlstsz = param[0];
	if(chlstsz < 1)
		return -EINVAL;
	if(chlstsz >= ADC_CHANLST_MAX)
		return -EINVAL;
	copy_from_user(param, (void *)arg, (chlstsz * 2 + 1)*sizeof(unsigned long));

	spin_lock(&padc->lock);
	padc->chanlstln = 0;
	padc->chanidx = 0;
	dm = padc->waterlevel / chlstsz;
	for(pos=0; pos<chlstsz; pos++) {
		pchan = &padc->chanlst[padc->chanlstln++];
		pchan->channel = ((param[pos*2+1]))&0xff;
		pchan->gain = ((param[pos*2+2]))&0xff;
		pchan->range = (((param[pos*2+2]))>>8)&0x0f;
		pchan->reference = (((param[pos*2+2]))>>12)&0x0f;
		if(pchan->gain > 7) pchan->gain = 7;
		if(pchan->range > 3) pchan->range = 3;
		if(pchan->reference > 1) pchan->reference = 1;
	}
	for(pos=0; pos<dm; pos++) {
		adc_progchanlist(&pdev->adc);
	}
	spin_unlock(&padc->lock);
	return 0;
}

/**
 * End of File
*/
