/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * isr.h -- Interrupt Service Routine interface header file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include <linux/dma-mapping.h>
#include "counter.h"
#include "adc.h"
#include "pci703def.h"
#include "dac.h"
#include "caldac.h"
#include "eeprom.h"
#include "isr.h"
#include "pci703.h"

static void dma_dump( pIrq_t pirq )
{
	unsigned long regval = readl(pirq->buscontrol);
	drv_err("%s: Error bit %ld", DEVICE, regval & DMA_ERROR_BIT);
	drv_err("%s: Done bit %ld", DEVICE, regval & DMA_DONE_BIT);
	drv_err("%s: Direction bit %ld", DEVICE, regval & DMA_DIR_BIT);
	drv_err("%s: Request bit %ld", DEVICE, regval & DMA_REQUEST_BIT);
	drv_err("%s: DMA Active bit %ld", DEVICE, regval & DMA_ACTIVE_BIT);
	drv_err("%s: Bytes xfred %ld", DEVICE, DMA_BUF_SIZE - ((regval & ~(DMA_XFR_MASK))>>16) );
}

int dma_test( pIrq_t pirq )
{
	unsigned long i;
	unsigned long val;
	unsigned long to = 1000000;	

	for(i=0; i<512; i++) {
		writel(16383 - i, pirq->datafifo);
	}

	val = readl(pirq->buscontrol) & ~( DMA_ERROR_BIT |
					DMA_DONE_BIT |
					DMA_ENABLE_BIT |
					DMA_DIR_BIT |
					DMA_XFR_MASK |
					DMA_BURST_MASK );
	rmb();
	val |= ( DMA_REQUEST_BIT | DMA_ENABLE_BIT | DMA_DIR_BIT * DMA_DIR_TOPCI | 
		    (DMA_BUF_SIZE << 16) | (0 << 29) );
	writel(val, pirq->buscontrol);	
	
	do {
		val = readl(pirq->buscontrol);
		to--;
	}while(!(val&DMA_DONE_BIT) && !(val&DMA_ERROR_BIT) && (to));

	if((!to) | (val&DMA_ERROR_BIT)) {
		drv_err("%s: DMA error occured", DEVICE);
		return -EIO;
	}

	for(i=0; i<512; i++) {
		if(pirq->dma_virt[i] != (16383 - i) ) {
			drv_err("%s: DMA data value test failed on %ld, got %ld.", DEVICE, i, pirq->dma_virt[i]);
			dma_dump(pirq);
			return -EIO;
		}
	}	
	return 0;
}


static void irq_off( pIrq_t pirq )
{
	unsigned long regval;
	regval = readl(pirq->control) & 0xfffffffe;
	writel(regval, pirq->control);	
	regval = readl(pirq->buscontrol) & ~( INT_ENABLE_BIT | INT_STATUS_BIT | DMA_ENABLE_BIT );
	writel(regval | INT_STATUS_BIT, pirq->buscontrol);	
}

static void irq_on( pIrq_t pirq )
{
	unsigned long regval;
	regval = readl(pirq->control) | 0x00000001;
	writel(regval, pirq->control);	
	regval = readl(pirq->buscontrol);
	writel(regval | INT_STATUS_BIT | INT_ENABLE_BIT, pirq->buscontrol);
}

static irqreturn_t dev_isr( int irq, void *dev_id, struct pt_regs *reg )
{
	pIrq_t pirq = (pIrq_t)dev_id;
	unsigned long dmareg = readl(pirq->buscontrol);
	unsigned long irqreg = readl(pirq->control);
	
	if(!(dmareg & INT_STATUS_BIT)) 
		return 0;
	
	writel(dmareg | INT_STATUS_BIT, pirq->buscontrol);
		
	if((dmareg & DMA_DONE_BIT) && !(dmareg & DMA_ERROR_BIT)) {
	
		if(adc_dmatobuf(pirq->pbuffer, pirq->dma_virt, pirq->waterlevel)) {		
			drv_err("%s: Stopping process.", DEVICE);
			dmareg &= ~( DMA_ERROR_BIT |
					DMA_DONE_BIT |
					DMA_ENABLE_BIT |
					DMA_DIR_BIT |
					DMA_XFR_MASK |
					DMA_BURST_MASK );		
			writel(dmareg, pirq->buscontrol);					
			return -EIO;
		}
	}
	else if(dmareg & DMA_ERROR_BIT) {
		drv_err("%s: DMA Error occured. Stopping process.", DEVICE);
		dma_dump(pirq);
		return -EIO;
	}		
		
	if(irqreg & IRQ_CHANLIST_DONE) {
		dmareg &= ~( DMA_ERROR_BIT |
			DMA_DONE_BIT |
			DMA_ENABLE_BIT |
			DMA_DIR_BIT |
			DMA_XFR_MASK |
			DMA_BURST_MASK );
			
		dmareg |= DMA_REQUEST_BIT | DMA_ENABLE_BIT | DMA_DIR_BIT * DMA_DIR_TOPCI | 
			DMA_BUF_SIZE << 16 | 0 << 29 | INT_STATUS_BIT | INT_ENABLE_BIT;
		    
		writel(dmareg, pirq->buscontrol);
		
		irqreg &= ~STAT_CHANLIST_DONE;
		writel(irqreg, pirq->control);
		
	}	
	return IRQ_HANDLED;
}

void irq_reset( pIrq_t pirq )
{
	unsigned long dmareg = readl(pirq->buscontrol);
	unsigned long irqreg = readl(pirq->control);
	
	dmareg &= ~( DMA_ERROR_BIT |
			DMA_DONE_BIT |
			DMA_ENABLE_BIT |
			DMA_DIR_BIT |
			DMA_XFR_MASK |
			DMA_BURST_MASK );
			
	dmareg |= INT_STATUS_BIT | INT_ENABLE_BIT;
		    
	writel(dmareg, pirq->buscontrol);
	irqreg &= ~( STAT_CHANLIST_DONE |
			STAT_CNT2_EXPIRED );	
	writel(irqreg, pirq->control);
}

int irq_enable( pIrq_t pirq )
{
	if(pirq->status == IRQ_STATUS_BUSY) {		
		return -EBUSY;	 
	}
	
	if( request_irq( pirq->pci_device->irq, dev_isr, SA_SHIRQ,
			DEVICE, (void*)pirq) ) {
			drv_err("%s: Failed to allocate irq." ,DEVICE);
			return -EIO;
	}	
	
	pirq->status = IRQ_STATUS_BUSY;	
	pirq->dma_size = DMA_BUF_SIZE;
	pirq->dma_virt = (unsigned long *)
			dma_alloc_coherent( &pirq->pci_device->dev,
						pirq->dma_size,
						&pirq->dma_addr,
						GFP_KERNEL );
	if(pirq->dma_virt == NULL) {		
		drv_err("%s: Failed to allocate consistent DMA memory.", DEVICE);
		free_irq(pirq->pci_device->irq, pirq);
		return -ENOMEM;
	}
	
	writel( pirq->dma_addr, pirq->buscontrol - 0x00000008);
	writel( 0x00000000, pirq->buscontrol - 0x00000004);	
	irq_on(pirq);
	return 0;
}

void irq_disable( pIrq_t pirq )
{
	irq_off(pirq);		
	if(pirq->status != IRQ_STATUS_BUSY) {
		return;
	}
	pirq->status = IRQ_STATUS_IDLE;
	if(pirq->dma_virt != NULL)
		dma_free_coherent( &pirq->pci_device->dev, pirq->dma_size,
				     pirq->dma_virt, pirq->dma_addr);
	free_irq(pirq->pci_device->irq, pirq);
}

void irq_init( pIrq_t pirq, void *control, void *buscontrol,
		      void *datafifo, pADCBuffer_t pbuffer)
{
	spin_lock_init(&pirq->lock);
	spin_lock(&pirq->lock);
	pirq->control = control;
	pirq->buscontrol = buscontrol;
	pirq->datafifo = datafifo;
	pirq->pbuffer = pbuffer;
	pirq->mode = MODE_PIO;	
	irq_off(pirq);
	spin_unlock(&pirq->lock);
}

void irq_uninit( pIrq_t pirq )
{	
	irq_disable(pirq);
}

/**
 * End of File
 */
