/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * isr.h -- Interrupt Service Routine & DMA related functions.
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#ifndef _ISR_H_
#define _ISR_H_

#define DMA_ERROR_BIT 0x00000001
#define DMA_DONE_BIT 0x00000004
#define DMA_DIR_BIT 0x00000008
#define DMA_DIR_TOPCI 0
#define DMA_DIR_FROMPCI 1
#define DMA_REQUEST_BIT 0x00000010
#define DMA_ENABLE_BIT 0x00000020
#define DMA_ACTIVE_BIT 0x00000040
#define INT_STATUS_BIT 0x00000100
#define INT_ENABLE_BIT 0x00000200
#define DMA_XFR_MASK 0x0fff0000
#define DMA_BURST_MASK 0xe0000000

#define IRQ_STATUS_IDLE 0
#define IRQ_STATUS_BUSY 1
#define STAT_CHANLIST_DONE 0x00000004
#define IRQ_CHANLIST_DONE 0x00000010
#define STAT_CNT2_EXPIRED 0x00008000

#define IRQ_SRC_ADCEMPTY 2
#define IRQ_SRC_DACEMPTY0 3
#define IRQ_SRC_DACHALF0 4
#define IRQ_SRC_DACEMPTY1 5
#define IRQ_SRC_DACHALF1 6
#define IRQ_SRC_CNT0 7
#define IRQ_SRC_CNT1 8
#define IRQ_SRC_CNT2 9

#define MODE_PIO 0
#define MODE_DMA 1

#define DMA_BUF_SIZE 2048

struct Irq_t
{
	spinlock_t lock;
	void *buscontrol;
	void *control;
	void *datafifo;
	dma_addr_t dma_addr;
	unsigned long *dma_virt;
	unsigned long dma_size;
	int dma_status;
	int mode;
	struct pci_dev *pci_device;
	unsigned char status;
	unsigned short waterlevel;
	pADCBuffer_t pbuffer;
};
#define pIrq_t struct Irq_t*

void irq_init( pIrq_t pirq, void *control, void *buscontrol,
		void *datafifo, pADCBuffer_t pbuffer );
void irq_uninit( pIrq_t pirq );
int irq_enable( pIrq_t pirq );
void irq_disable( pIrq_t pirq );
void irq_reset( pIrq_t pirq );
int dma_test( pIrq_t pirq );

#endif /*_ISR_H_*/
/**
 * End of File
 */
