/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * ioctl.c -- I/O control source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "eeprom.h"
#include "pci730inc.h"
#include "pci730.h"
#include "ioctl.h"
#include "version.h"

/**
 * Helper functions
 */

long ioctl_getunread( pBoardDevice_t pdev )
{
	unsigned long flags;
	long unread;
	spin_lock_irqsave(&pdev->buf_lock, flags);
	unread = pdev->buf_unread;
	spin_unlock_irqrestore(&pdev->buf_lock, flags);
	return unread;
}

void ioctl_resetbuffer( pBoardDevice_t pdev )
{
	pdev->buf_head = 0;
	pdev->buf_tail = 0;
	pdev->buf_over = 0;
	pdev->buf_unread = 0;
	pdev->adsetup.enable = DISABLE;
	pdev->adsetup.resetfifo = 1;
	pdev->adsetup.mode = 0;
	ioctl_adcfgsetup(pdev);
	pdev->adsetup.resetfifo = 0;
}

void ioctl_adcfgsetup( pBoardDevice_t pdev )
{
	unsigned long cfgval;
	cfgval = (unsigned long)( (pdev->adsetup.mode << 12) |
			(pdev->adsetup.ADconvert << 11) |
			(pdev->adsetup.ints_enable << 10) |
			(pdev->adsetup.fifostatus << 9) |
			(pdev->adsetup.trigmode << 8) |
			(pdev->adsetup.listsize << 4) |
			(pdev->adsetup.fifomode << 3) |
			(pdev->adsetup.resetfifo << 2) |
			(pdev->adsetup.clkmode << 1) |
			(pdev->adsetup.enable) );
	writel(cfgval, pdev->ptrmem + PCI730_REG_AD_SETUP);
}

long ioctl_bintovolts( pBoardDevice_t pdev, unsigned long binval,
					   unsigned long gain)
{
	long uv, bin;
	long val;

	bin = binval;
	if(bin >= 0x2000) {
		bin&=0x3fff;
		bin-=0x4000;
	}

	bin += (pdev->devinfo.adcoffset - 500);
	uv = bin * 305;
	uv += (pdev->devinfo.AMPoffset - 5000);

	if(uv < 0)
		val = uv / -10;
	else
		val = uv / 10;

	if(pdev->devinfo.adcgain[gain] < 500)
		val = ( (val * pdev->devinfo.adcgain[gain]) / (pdev->devinfo.adcgain[gain] + 1)) * (1 << gain);
	else
		val = ((val * (1000 - pdev->devinfo.adcgain[gain])) / (999 - pdev->devinfo.adcgain[gain])) * (1 << gain);

	if(uv < 0)
		uv = val * -10;
	else
		uv = val * 10;
	return uv;
}

void ioctl_8255cfg( pBoardDevice_t pdev )
{
	unsigned long cfg;
	unsigned long CVAL[4] = {0,9,8,1};
	cfg = 	(1 << 7) |
			(pdev->ppi.mode[0] << 5) |
			(pdev->ppi.io[0] << 4) |
			(pdev->ppi.mode[1] << 2) |
			(pdev->ppi.io[1] << 1) |
			(CVAL[pdev->ppi.io[2]]);
	writel(cfg, pdev->ptrmem + PCI730_REG_DIO_OFFSET + 0x0c);
}

int ioctl_getporttype( pBoardDevice_t pdev, int port )
{
	if(pdev->devinfo.diochan > 0)
		return DIOINOROUT;
	else
		return -1;
}

int ioctl_getportwidth( pBoardDevice_t pdev, int port )
{
	if(pdev->devinfo.diochan > 0)
		return 8l;
	else
		return 0;
}

void ioctl_8254cfg( pBoardDevice_t pdev, int ct, int mode, int bcd )
{
	unsigned long cfgval;
	cfgval = (unsigned long)( (ct << 6) | (CMD_RW_LSBMSB << 4) | (mode << 1) | bcd);
	writel(cfgval, pdev->ptrmem + PCI730_REG_CT_OFFSET + 0x0c);
}

void ioctl_ctregcfg( pBoardDevice_t pdev )
{
	unsigned long cfgval;
	cfgval = (unsigned long)( (pdev->ct.cntval << 16) |
								(pdev->ct.mask[2] << 8) |
								(pdev->ct.mask[1] << 7) |
								(pdev->ct.mask[0] << 6) |
								(pdev->ct.status[2] << 5) |
								(pdev->ct.status[1] << 4) |
								(pdev->ct.status[0] << 3) |
								(pdev->ct.gate[2] << 2) |
								(pdev->ct.gate[1] << 1) |
								(pdev->ct.gate[0]) );
	writel(cfgval, pdev->ptrmem + PCI730_REG_CT_CFG);
}

long ioctl_adcgetdata( pBoardDevice_t pdev, long *buffer, unsigned long samples )
{
	unsigned long tail, unread, gain, i, flags;

	spin_lock_irqsave(&pdev->buf_lock, flags);
	tail = pdev->buf_tail;
	unread = pdev->buf_unread;
	spin_unlock_irqrestore(&pdev->buf_lock, flags);

	if(samples > unread)
		return -EINVAL;

	for(i=0; i<samples; i++) {
		gain = pdev->adchan[pdev->buf_pos++].gain;
		if(pdev->buf_pos > pdev->adsetup.listsize)
			pdev->buf_pos = 0;
		*(buffer + i) = ioctl_bintovolts(pdev, *(pdev->buffer + tail), gain);
		if(++tail == BUFFERSIZE)
			tail = 0;
	}

	spin_lock_irqsave(&pdev->buf_lock, flags);
	pdev->buf_tail = tail;
	pdev->buf_unread -= samples;
	spin_unlock_irqrestore(&pdev->buf_lock, flags);

	return samples;
}


/**
 * I/O control exported functions
 */
int ioctl_query( pBoardDevice_t pdev, unsigned long arg )
{
#define NUMADCRANGES 3
#define NUMADCMODES 2
#define NUMADCCLKS 3
#define NUMADCTRIGS 2
#define NUMADCTRIGMODES 0

	unsigned long param[2];
	const long ADCRANGES[NUMADCRANGES] = { 5000000, 10000000, 20000000 };
	const long ADCCLKS[NUMADCCLKS] = { AD_CLK_INTERNAL, AD_CLK_EXTERNAL };
	const long ADCMODES[NUMADCMODES] = { BI_SINGLE, BI_DIFF };
	const long ADCTRIGS[NUMADCTRIGS] = { AD_TRIG_INTERNAL,AD_TRIG_EXTERNAL };

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	switch(param[0]) {
	case BRDSERIALNO:
		param[0] = pdev->devinfo.serialnumber;
		break;
	case BRDREV:
		param[0] = pdev->devinfo.revision;
		break;
	case BRDTYPE:
		param[0] = pdev->devinfo.boardtype;
		break;
	case BRDYEAR:
		param[0] = pdev->devinfo.year;
		break;
	case BRDMONTH:
		param[0] = pdev->devinfo.month;
		break;
	case BRDDAY:
		param[0] = pdev->devinfo.day;
		break;
	case BRDBUSTYPE:
		param[0] = PC_BUS_PCI;
		break;
	case DRVMAJOR:
		param[0] = (PCI730DRV_VERSION>>16)&0xff;
		break;
	case DRVMINOR:
		param[0] = (PCI730DRV_VERSION>>8)&0xff;
		break;
	case DRVBUILD:
		param[0] = PCI730DRV_VERSION&0xff;
		break;
	case ADNUMCHAN:
		param[0] = pdev->devinfo.adcchan;
		break;
	case ADUNREAD:
		param[0] = ioctl_getunread(pdev);
		break;
	case ADMAXFREQ:
		param[0] = 100000;
		break;
	case ADFIFOSIZE:
		param[0] = 2048;
		break;
	case ADBUFFALLOC:
		if(pdev->buffer != NULL)
			param[0] = 1;
		else
			param[0] = 0;
		break;
	case ADCLRBUF:
		ioctl_resetbuffer(pdev);
		param[0] = 0;
		break;
	case ADIRQLEVEL:
		pdev->waterlevel = param[1];
		param[0] = 0;
		break;
	case ADNUMRANGES:
		param[0] = NUMADCRANGES;
		break;
	case ADRANGES:
		if(param[1] >= NUMADCRANGES)
			return -EINVAL;
		param[0] = ADCRANGES[param[1]];
		break;
	case ADNUMMODES:
		param[0] = NUMADCMODES;
		break;
	case ADMODES:
		if(param[1] >= NUMADCMODES)
			return -EINVAL;
		param[0] = ADCMODES[param[1]];
		break;
	case ADNUMGAINS:
		param[0] = 0;
		break;
	case ADGAINS:
		param[0] = 0;
		break;
	case ADNUMTRIG:
		param[0] = NUMADCTRIGS;
		break;
	case ADTRIG:
		if(param[1] >= NUMADCTRIGS)
			return -EINVAL;
		param[0] = ADCTRIGS[param[1]];
		break;
	case ADNUMTRIGMODE:
		param[0] = NUMADCTRIGMODES;
		break;
	case ADTRIGMODE:
		return -EINVAL;
	case ADNUMCLK:
		param[0] = NUMADCCLKS;
		break;
	case ADCLK:
		if(param[1] >= NUMADCCLKS)
			return -EINVAL;
		param[0] = ADCCLKS[param[1]];
		break;
	case ADSETDATAMODE:
		param[0] = 0;
		break;
	case ADREFGNDPIN:
		param[0] = 1;
		break;
	case ADNUMGATES:
		param[0] = 0;
		break;
	case ADGATE:
		param[0] = 0;
		break;
	case ADDMATEST:
		return -EINVAL;
	case DANUMCHAN:
		param[0] = pdev->devinfo.dacchan;
		break;
	case CTNUM:
		param[0] = pdev->devinfo.cntchan;
		break;
	case DIONUMPORT:
		param[0] = pdev->devinfo.diochan;
		break;
	case DIOQRYPORT:
		param[0] = DIOINOROUT;
		break;
	case DIOPORTWIDTH:
		param[0] = 1;
		break;
	case INTNUMSRC:
		param[0] = 0;
		break;
	default:
		param[0] = -1;
	}
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return 0;
}

int ioctl_diowrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,value;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	port = param[0];
	value = param[1];

	if(pdev->devinfo.diochan == 0)
		return -EINVAL;

	if(pdev->ppi.io[port] != DIOOUT) {
		pdev->ppi.io[port] = DIOOUT;
		ioctl_8255cfg(pdev);
	}
	writel(param[1] & 0xff, pdev->ptrmem + PCI730_REG_DIO_OFFSET + (port * 4));

	return 0;
}

int ioctl_dioread( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,value;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port = param[0];

    if(pdev->devinfo.diochan == 0)
		return -EINVAL;

	if(pdev->ppi.io[port] != DIOIN) {
		pdev->ppi.io[port] = DIOIN;
		ioctl_8255cfg(pdev);
	}
	value = readl(pdev->ptrmem + PCI730_REG_DIO_OFFSET + (port * 4)) & 0xff;

	param[0] = value;
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return 0;
}


int ioctl_ctwrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,value;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	port = param[0];
	value = param[1] & 0xffffl;

	if(port >= pdev->devinfo.cntchan)
		return -EINVAL;

	if(port < 3) {
		writel(value & 0xff, pdev->ptrmem + PCI730_REG_CT_OFFSET + port * 4);
		writel( (value>>8) & 0xff, pdev->ptrmem + PCI730_REG_CT_OFFSET + port * 4);
	}
	else {
		pdev->ct.cntval = value;
		pdev->ct.status[0] = 1;
		ioctl_ctregcfg(pdev);
	}

	return 0;
}

int ioctl_ctread( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,value;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port = param[0];

	if(port >= pdev->devinfo.cntchan)
		return -EINVAL;

	if(port < 3) {
		value = readl(pdev->ptrmem + PCI730_REG_CT_OFFSET + (port * 4)) & 0xff;
		value |= (readl(pdev->ptrmem + PCI730_REG_CT_OFFSET + (port * 4)) & 0xff) << 8;
	}
	else {
		value = readl(pdev->ptrmem + PCI730_REG_CT_CFG) >> 16;
	}
	value &= 0xffff;

	param[0] = value;
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return 0;
}


int ioctl_ctcfg( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[5],port,mode,bcd,clksrc,gatesrc;

	copy_from_user(param, (void *)arg, 5 * sizeof(unsigned long));
	port = param[0];
	mode = param[1];
	bcd = param[2];
	clksrc = param[3];      //not used
	gatesrc = param[4];     //not used

	if(port >= pdev->devinfo.cntchan)
		return -EINVAL;

	if(port < 3) {
		ioctl_8254cfg(pdev,port,mode,bcd);
	}
	else
		return -EINVAL;
	return 0;
}

int ioctl_ctsoftgate( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,mode;
	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	port = param[0];
	mode = param[1];

	if((port > 2) || (mode > 1))
		return -EINVAL;

	pdev->ct.gate[port] = mode;
	ioctl_ctregcfg(pdev);

	return 0;
}

int ioctl_dawrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port;
	long value,sign;
	unsigned long binval,regval,voltage,mult = 1;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	port = param[0];
	value = param[1];

	if(port >= pdev->devinfo.dacchan)
		return -EINVAL;
	if((value > 10000000) || (value < -10000000))
		return -EINVAL;
	if( (readl(pdev->ptrmem + PCI730_REG_DAC_CONTROL) & 0x80000) != 0x80000 )
		return -EBUSY;

	if(value < 0) {
		sign = -1;
		value *= -1;
	}
	else
		sign=1;

	voltage = (unsigned long)value;
	voltage /= 10;

	if(pdev->devinfo.dacgain[port] < 5000) {
		mult = pdev->devinfo.dacgain[port] * 1000;
		mult /= (pdev->devinfo.dacgain[port] + 1);
	}
	else {
		mult = (10000 - pdev->devinfo.dacgain[port]) * 1000;
		mult /= (9999 - pdev->devinfo.dacgain[port]);
	}

	if(mult == 0)
		mult=1;

	voltage *= mult;
	value = (long)voltage;
	value *= sign;
	value /= 122070;
	mult = 7192 + pdev->devinfo.dacoffset[port];
	value += mult;

	if(value < 0)
		value = 0;
	if(value > 16383)
		value = 16383;

	binval = (unsigned long)(value);
	regval = (0 << 18) | (port << 16) | (binval << 2);
	writel(regval, pdev->ptrmem + PCI730_REG_DAC_CONTROL);

	return 0;
}

int ioctl_intconfig( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],src,mode;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	src = param[0];
	mode = param[1];

	if((src >= pdev->devinfo.cntchan) || (mode > 1))
		return -EINVAL;
	pdev->ct.mask[src] = mode;

	ioctl_ctregcfg(pdev);

	return 0;
}


int ioctl_intenable( pBoardDevice_t pdev, unsigned long arg )
{
	pdev->int_received = 0;
	pdev->ct.ints = ENABLE;
	return 0;
}

int ioctl_intdisable( pBoardDevice_t pdev, unsigned long arg )
{
	pdev->ct.ints = DISABLE;
    return 0;
}

int ioctl_intwait( pBoardDevice_t pdev, unsigned long arg )
{
	if(pdev->int_wait)
		return -EBUSY;
	pdev->int_wait = 1;
	init_waitqueue_head(&pdev->wq);
	wait_event_interruptible(pdev->wq, 1);
	pdev->int_wait = 0;
	return 0;
}

int ioctl_intrelease( pBoardDevice_t pdev, unsigned long arg )
{
	if(pdev->int_wait)
		wake_up_interruptible(&pdev->wq);
	return 0;
}

int ioctl_adsingle( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],gain,chan;
	unsigned long cfgval, binval;
	long uv;

	if(pdev->adc_busy)
		return -EBUSY;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	chan = param[0];
	gain = param[1];

	if(chan > 15)
		return -EINVAL;
	if(gain > 2)
		return -EINVAL;

	ioctl_resetbuffer(pdev);
	cfgval = (unsigned long)( chan | (pdev->adchan[0].mode << 5) | (gain<<6));
	writel(cfgval, pdev->ptrmem + PCI730_REG_ADC_LIST);
	udelay(50);

	pdev->adsetup.enable = ENABLE;
	pdev->adsetup.clkmode = 1;
	pdev->adsetup.fifomode = 0;
	pdev->adsetup.listsize = 1;
	pdev->adsetup.trigmode = 0;
	pdev->adsetup.ints_enable = DISABLE;
	pdev->adsetup.ADconvert = 1;

	ioctl_adcfgsetup(pdev);
	udelay(50);

	binval = (unsigned long)(readl(pdev->ptrmem + PCI730_REG_AD_FIFO));

	uv = ioctl_bintovolts(pdev, binval, gain);
	copy_to_user((void *)arg, &uv, sizeof(long));
	return 0;
}

int ioctl_adconfig( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],range;

	if(pdev->adc_busy) {
		drv_err("pci730: device is busy");
		return -EBUSY;
	}

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	range = param[0];

	if(range > 1) {
		drv_err("pci730: range value invalid");
		return -EINVAL;
	}

	ioctl_resetbuffer(pdev);

	pdev->adchan[0].mode = range;
	return 0;
}

int ioctl_adstop( pBoardDevice_t pdev, unsigned long arg )
{
	pdev->adc_busy = 0;
	pdev->adsetup.enable = DISABLE;
	pdev->adsetup.ints_enable = DISABLE;
	ioctl_adcfgsetup(pdev);
	return 0;
}

int ioctl_adstart( pBoardDevice_t pdev, unsigned long arg )
{
	if(pdev->adc_busy)
		return -EBUSY;
	writel(pdev->waterlevel, pdev->ptrmem + PCI730_REG_AD_WATERMARK);
	pdev->adc_busy = 1;
	pdev->adsetup.enable = ENABLE;
	pdev->adsetup.ints_enable = ENABLE;
	pdev->buf_head = 0;
	pdev->buf_tail = 0;
	pdev->buf_unread = 0;
	pdev->buf_over = 0;
	ioctl_adcfgsetup(pdev);
	return 0;
}

int ioctl_adclksrc(pBoardDevice_t pdev, unsigned long arg)
{
	unsigned long param[1], clksrc;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	clksrc = param[0];
	pdev->adsetup.clkmode = 0;
	pdev->adsetup.trigmode = (clksrc>>8) & 0x01;
	return 0;
}

int ioctl_adfreq(pBoardDevice_t pdev, unsigned long arg)
{
#define MASTER_CLOCK_20MHZ 20000000l
	unsigned long param[1], freq, val;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	freq = param[0];

	if(freq > 100000 || freq < 10)
		return -EINVAL;

	val = MASTER_CLOCK_20MHZ / freq;
	freq = MASTER_CLOCK_20MHZ / val;

	writel(val,pdev->ptrmem + PCI730_REG_AD_CT);
	ioctl_adcfgsetup(pdev);

	copy_to_user((void *)arg, &freq, sizeof(unsigned long));
	return 0;
}

int ioctl_adchlst( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[33], chlst, cnt, chan, gain, cfgval, mode;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	chlst = param[0];

	if(chlst > 16) {
		drv_err("pci730: channel list length error");
		return -EINVAL;
	}

	copy_from_user(param, (void *)arg, (chlst * 2 + 1) * sizeof(unsigned long));

	pdev->adsetup.listsize = chlst - 1;

	for(cnt=0; cnt<chlst; cnt++) {
		chan = param[cnt * 2 + 1];
		if(chan > 15) {
			drv_err("pci730: channel no error");
			return -EINVAL;
		}

		gain = (param[cnt * 2 + 2]) & 0x03;
		mode = (param[cnt * 2 + 2] >> 8) & 0x01;

		if((gain > 2) || (mode>1)) {
			drv_err("pci730: gain or mode error");
			return -EINVAL;
		}

		pdev->adchan[cnt].chan = chan;
		pdev->adchan[cnt].mode = mode;
		pdev->adchan[cnt].gain = gain;
		cfgval = ( (gain << 6) | (mode << 5) | chan );

		writel(cfgval, pdev->ptrmem + PCI730_REG_ADC_LIST + cnt * 4);
	}

	pdev->buf_pos = 0;
	return 0;
}

/**
 * End of File
*/
