/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * isr.c -- Interrupt Service Routine source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "eeprom.h"
#include "pci730inc.h"
#include "pci730.h"
#include "ioctl.h"
#include "isr.h"

static irqreturn_t dev_isr( int irq, void *dev_id, struct pt_regs *reg )
{

	unsigned long s, regval, head, unread;
	pBoardDevice_t pdev = (pBoardDevice_t)dev_id;

	/* counters TC*/
	regval = readl(pdev->ptrmem + PCI730_REG_CT_CFG);
	if(regval & INT_SRC_CTALL) {
		pdev->int_trigsrc = (regval & INT_SRC_CTALL) >> 3;
		writel( regval | INT_SRC_CTALL, pdev->ptrmem + PCI730_REG_CT_CFG);
		spin_lock(&pdev->int_lock);
		if(pdev->int_wait)
			wake_up_interruptible(&pdev->wq);
		spin_unlock(&pdev->int_lock);
	}

	/* A/D fifo */
	regval = readl(pdev->ptrmem + PCI730_REG_AD_SETUP);
	if( regval & INT_SRC_FIFO) {
		spin_lock(&pdev->buf_lock);
		head = pdev->buf_head;
		unread = pdev->buf_unread;
		spin_unlock(&pdev->buf_lock);

		s = pdev->waterlevel;
		do {
			*(pdev->buffer + head) = readl(pdev->ptrmem + PCI730_REG_AD_FIFO);
			if(++head == BUFFERSIZE)
				head = 0;
			if(++unread == BUFFERSIZE) {
				drv_err("%s: Buffer overrun.", DEVICE);
				pdev->buf_over = 1;
				pdev->adc_busy = 0;
				pdev->adsetup.enable = DISABLE;
				pdev->adsetup.ints_enable = DISABLE;
				ioctl_adcfgsetup(pdev);
			}
		} while( (s--) && !(pdev->buf_over) );

		spin_lock(&pdev->buf_lock);
		pdev->buf_head = head;
		pdev->buf_unread += pdev->waterlevel  - s;
		spin_unlock(&pdev->buf_lock);
	}

	return IRQ_HANDLED;
}


int irq_enable( pBoardDevice_t pdev )
{
	if(pdev->int_status == IRQ_STATUS_BUSY) {
		return -EIO;
	}

	if( request_irq( pdev->pci_device->irq, dev_isr, SA_SHIRQ,
		DEVICE, (void*)pdev) ) {
			return -EIO;
	}

	pdev->int_status = IRQ_STATUS_BUSY;
	return EDRE_OK;
}

void irq_disable( pBoardDevice_t pdev )
{
	if(pdev->int_status == IRQ_STATUS_BUSY) {
		free_irq(pdev->pci_device->irq, pdev);
	}
	pdev->int_status = IRQ_STATUS_IDLE;
}


/**
 * End of File
 */
