/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * pci730.h -- PCI730 main header file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#ifndef _PCI730_H_
#define _PCI730_H_

#define BUFFERSIZE 1048576
#define FIFOSIZE 2048
#define HALFFIFO 1024
#define READSZ 1000
#define DOUBLE_BUFFER_SIZE 8192
#define DEVICE "pci730drv"

struct Intel8255_t {
	int mode[2];
	int io[3];
};
#define pIntel8255_t struct Intel8255_t*

//counter timer data
struct Intel8254_t {
	int	gate[3];
	int status[3];
	int	mask[3];
	unsigned long ints;
	unsigned long cntval;
};
#define pIntel8254_t struct Intel8254_t*

struct DeviceInfo_t {
	unsigned long	serialnumber;
	unsigned short boardtype;
	unsigned short revision;
	unsigned short year, month, day;
	unsigned short adcchan, dacchan, diochan, cntchan, miochan, irqchan;
	unsigned short hcport,optoport;
	unsigned long adcfreq, dacfreq;
	unsigned long dacoffset[4];
	unsigned long dacgain[4];
	unsigned long adcoffset;
	unsigned long AMPoffset;
	unsigned long adcgain[4];
};
#define pDeviceInfo_t struct DeviceInfo_t*

struct ADSetup_t {
	unsigned short	enable;
	unsigned short	clkmode;
	unsigned short	resetfifo;
	unsigned short	fifomode;
	unsigned short	listsize;
	unsigned short	trigmode;
	unsigned short	fifostatus;
	unsigned short	ints_enable;
	unsigned short	ADconvert;
	unsigned short	mode;
};
#define pADSetup_t struct ADSetup_t*

struct ADChannel_t {
	unsigned char chan;
	unsigned char mode;
	unsigned char gain;
};
#define pADChannel_t struct ADChannel_t*

struct DblBuffer_t {
	void *data;
	unsigned short size;
};
#define pDblBuffer_t struct DblBuffer_t*

struct BoardDevice_t {
	spinlock_t lock;
	struct pci_dev *pci_device;
	struct class_device *clsdev;
	int minor;
	wait_queue_head_t wq;
	spinlock_t int_lock;
	unsigned long int_received;
	unsigned char int_wait;
	unsigned char int_status;
	unsigned long int_trigsrc;
	unsigned char adc_int;
	unsigned char adc_busy;

	spinlock_t buf_lock;
	unsigned long buf_head;
	unsigned long buf_tail;
	unsigned long buf_pos;
	unsigned long buf_over;
	unsigned long buf_unread;
	unsigned short *buffer;
	unsigned short waterlevel;

	unsigned int membase;
	void* ptrmem;
	struct ADChannel_t adchan[16];
	struct ADSetup_t adsetup;
	struct Intel8255_t ppi;
	struct Intel8254_t ct;
	struct DeviceInfo_t devinfo;
	struct Eeprom_t eeprom;
	struct DblBuffer_t dblbuffer;
	struct kref kref;
};
#define pBoardDevice_t struct BoardDevice_t*

#endif /* _PCI730_H_ */

/**
 * End of File
 */
