/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * da8802.c -- Calibration D/A soure file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "da8802.h"
 
void caldac_init( pCalDac_t pdac, void *Register )
{
	pdac->reg = Register;
	caldac_reset(pdac);
}

long caldac_write( pCalDac_t pdac, long pos, long val )
{
		long i, bm;
		val &= 0xff;
		
		if((pos > 12) || (pos < 0))
			return -1;
			
		caldac_enable(pdac);
		
		//dac - 4 bits
		bm = 0x8;
		for(i=0; i<4; i++) {
			caldac_writebit(pdac, bm&pos);
			bm >>= 1;
		}
		
		//value - 8 bits
		bm = 0x80;
		for(i=0; i<8; i++) {
			caldac_writebit(pdac, bm&val);
			bm >>= 1;
		}
	
		caldac_disable(pdac);
		return 0;
}

void caldac_writebit( pCalDac_t pdac, int val )
{
		unsigned long cntrl;
		cntrl = readl(pdac->reg) & (DAC_LOW | DAC_RST);
		
		if(val > 1)
			val = 1;

		writel(cntrl | (val * DAC_SDI), pdac->reg);
		udelay(2);
		writel(cntrl | (val * DAC_SDI) | DAC_CLK, pdac->reg);
		udelay(2);
		writel(cntrl | (val * DAC_SDI), pdac->reg);
		udelay(2);
}

void caldac_enable( pCalDac_t pdac )
{
		unsigned long cntrl;
		cntrl = readl(pdac->reg) & (DAC_LOW | DAC_RST);
		writel(cntrl, pdac->reg);
}

void caldac_disable( pCalDac_t pdac )
{
		unsigned long cntrl;
		cntrl = readl(pdac->reg) & (DAC_LOW | DAC_RST);
		writel(cntrl | DAC_CS, pdac->reg);
}

void caldac_reset( pCalDac_t pdac )
{
		unsigned long cntrl;
		cntrl = readl(pdac->reg) & DAC_LOW;
		writel(cntrl | DAC_RST, pdac->reg);
		udelay(2);
		caldac_disable(pdac);
}

/**
 * End of File
*/
