/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * dac.c -- D/A Interface source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "version.h"
#include "pci766inc.h" 
#include "ct.h"
#include "dac.h"  
 
void dac_init( pDAC_t pdac, pCounter_t pCt, void *RegChanList, void *RegFifoReset, void *RegWaterMark )
{
	spin_lock_init(&pdac->lock);
	pdac->regchanlist = RegChanList;
	pdac->regfiforeset = RegFifoReset;
	pdac->regwatermark = RegWaterMark;
	pdac->size = 0;
	pdac->startchan = 0;
	pdac->busy = 0;
	pdac->alloc = 0;
	pdac->load = 0;
	pdac->gate = 0;
	pdac->pct = pCt;
	dac_update(pdac);
	dac_configure(pdac, 0, 0, 0, 1000, DAC_MODE_LOOP);
	dac_fiforeset(pdac);
}

long dac_isbusy( pDAC_t pdac )
{
	long busy;
	spin_lock(&pdac->lock);
	busy = pdac->busy;
	spin_unlock(&pdac->lock);
	return busy;
}

void dac_update( pDAC_t pdac )
{
	unsigned long value = pdac->mode<<13 |
				pdac->load<<12 | 
				pdac->alloc<<6 | 
				pdac->startchan<<3 |
				pdac->size;
	writel(value, pdac->regchanlist);
}

void dac_fiforeset( pDAC_t pdac )
{
	spin_lock(&pdac->lock);
	pdac->mode = 0;
	pdac->alloc = 0;
	pdac->size = 0;
	dac_update(pdac);
	pdac->isconfig = 0;
	pdac->gate = 0;
	spin_unlock(&pdac->lock);	
	dac_configure(pdac, 0, 0, 0, 1000, DAC_MODE_LOOP);
	writel(0, pdac->regfiforeset);
}

void dac_configure( pDAC_t pdac, long src, long clk, long gate, 
			long freq, long cont )
{
#define CT_MODE_SQUAREWAVE 2
	spin_lock(&pdac->lock);
	pdac->load = src;
	ct_configure(pdac->pct, CT_MODE_SQUAREWAVE, clk, gate);
	ct_write(pdac->pct, freq);
	pdac->isconfig = 0;
	pdac->gate = gate;
	pdac->mode = cont;
	spin_unlock(&pdac->lock);
}

void dac_setalloc( pDAC_t pdac, long val )
{
	spin_lock(&pdac->lock);
	pdac->alloc = val;
	spin_unlock(&pdac->lock);
}

void dac_setsize( pDAC_t pdac, long val )
{
	spin_lock(&pdac->lock);
	pdac->size = val;
	spin_unlock(&pdac->lock);
}

void dac_setstartchan( pDAC_t pdac, long chan )
{
	spin_lock(&pdac->lock);
	pdac->startchan = chan;
	spin_unlock(&pdac->lock);
}

long dac_getmode( pDAC_t pdac )
{
	return pdac->mode;
}

void dac_enable( pDAC_t pdac )
{
	int i;
	spin_lock(&pdac->lock);
	pdac->size = 0;
	for(i=0; i<7; i++) {
		if( (pdac->alloc & (1<<i) ) > 0)
			pdac->size++;
	}
	dac_update(pdac);
	pdac->busy = true;
	ct_gate(pdac->pct, pdac->gate);
	spin_unlock(&pdac->lock);
}

void dac_disable( pDAC_t pdac )
{
	spin_lock(&pdac->lock);
	pdac->busy = false;
	ct_gate(pdac->pct, 0);
	writel(0, pdac->regchanlist);
	spin_unlock(&pdac->lock);
}

void dac_setwaterLevel( pDAC_t pdac, long value )
{
	spin_lock(&pdac->lock);
	pdac->waterlevel = value;
	writel(value, pdac->regwatermark);
	spin_unlock(&pdac->lock);
}

/**
 * End of File
 */
