/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * isr.c -- Interrupt Service Routine source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "version.h"
#include "buffer.h"
#include "isr.h"
 
void irq_enablefifo( pIsr_t pisr )
{
	pisr->mask &= ~IRQ_SRC_FFLEVEL;
	writel((INT_CFG_LEVEL1<<12), pisr->regcontrol);
	writel(pisr->mask, pisr->regmask);
}

void irq_disablefifo( pIsr_t pisr )
{
	pisr->mask |= IRQ_SRC_FFLEVEL;
	writel(pisr->mask, pisr->regmask);
}

static irqreturn_t dev_isr( int irq, void *dev_id, struct pt_regs *reg )
{
	pIsr_t pisr = (pIsr_t)dev_id;
	pBuffer_t pbuf = pisr->pbuffer;
	unsigned long dmareg = readl(pisr->regdma);
	unsigned long irqreg = readl(pisr->regstatus);

	if(!(dmareg & INT_STATUS_BIT)) 
		return -EIO;
	
	writel(dmareg | INT_STATUS_BIT, pisr->regdma);
	
	if(irqreg & IRQ_SRC_FFLEVEL) {
		if(buffer_fillfifo(pbuf)) {
			writel(dmareg & ~(INT_STATUS_BIT), pisr->regdma);
			return -EIO;
		}
		else {
			writel(0, pisr->regstatus);
			writel(dmareg | INT_STATUS_BIT | INT_ENABLE_BIT, pisr->regdma);
		}
	}
	else {
		err( "%s: Unknown interrupt.", DEVICE );
	}
	return IRQ_HANDLED;
}

int irq_enable( pIsr_t pisr )
{
	if(pisr->status) {
		err( "%s: Interrupt already enabled.", DEVICE );
		return -EBUSY;
	}
	
	if( request_irq( pisr->pci_device->irq, dev_isr, SA_SHIRQ,
			DEVICE, (void*)pisr) ) {
			err( "%s: Failed to allocate irq.", DEVICE );
			return -EIO;
	}
	
	pisr->status = IRQ_STATUS_BUSY;
	writel(0, pisr->regstatus);
	writel(0x000000ff, pisr->regstatus);
	writel(INT_STATUS_BIT | INT_ENABLE_BIT, pisr->regdma);
	return 0;
}

void irq_disable( pIsr_t pisr )
{
	writel(0x000000ff, pisr->regstatus);
	writel(INT_STATUS_BIT, pisr->regdma);
	
	if(pisr->status)
		free_irq(pisr->pci_device->irq, pisr);
		
	pisr->status = IRQ_STATUS_IDLE;
}

void irq_init( pIsr_t pisr, pBuffer_t pBuffer, void *RegControl, void *RegMask, void *RegStatus, void *RegDMA )
{
	pisr->regcontrol = RegControl;
	pisr->regmask = RegMask;
	pisr->regstatus = RegStatus;
	pisr->regdma = RegDMA;
	pisr->status = IRQ_STATUS_IDLE;	
	pisr->pbuffer = pBuffer;
	pisr->mask = 0xffff;	
	writel(pisr->mask, pisr->regmask);
	writel(INT_STATUS_BIT , pisr->regdma);
}


/**
 * End of File
 */
