/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * isr.h -- Interrupt Service Routine Interface Header File
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#ifndef _ISR_H_
#define _ISR_H_

#define IRQ_STATUS_BUSY 1
#define IRQ_STATUS_IDLE 0

#define INT_STATUS_BIT 0x00000100
#define INT_ENABLE_BIT 0x00000200
#define INT_MASK ~(INT_STATUS_BIT | INT_ENABLE_BIT)

/* bits defined as in the interrupt status register */
#define IRQ_SRC_TC0 0x000000001 /* counter 0 terminal count */
#define IRQ_SRC_TC1 0x000000002 /* counter 1 terminal count */
#define IRQ_SRC_DI0 0x000000004 /* digital input 0 */
#define IRQ_SRC_DI2 0x000000008 /* digital input 2 */
#define IRQ_SRC_DI3 0x000000010 /* digital input 4 */
#define IRQ_SRC_DI5 0x000000020 /* digital input 5 */
#define IRQ_SRC_FFLEVEL 0x000000040 /* fifo level less or equal to watermark */
#define IRQ_SRC_FFLEMPTY 0x000000080 /* fifo is empty */

/* interrupt configuration types */
#define INT_CFG_RISING 0
#define INT_CFG_FALLING 1
#define INT_CFG_LEVEL1 2
#define INT_CFG_LEVEL0 3

struct Isr_t {
	struct pci_dev *pci_device;
	pBuffer_t pbuffer;
	unsigned char status;
	unsigned short mask;
	void *regmask;
	void *regstatus;
	void *regcontrol;
	void *regdma;
};
#define pIsr_t struct Isr_t*

void irq_init( pIsr_t pisr, pBuffer_t pBuffer, void *RegControl, void *RegMask, void *RegStatus, void *RegDMA );
int irq_enable( pIsr_t pisr );
void irq_disable( pIsr_t pisr );
void irq_enablefifo( pIsr_t pisr );
void irq_disablefifo( pIsr_t pisr );

#endif /*_ISR_H_*/ 
/**
 * End of File
 */
