/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * ioctrl.c -- I/O Control source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "eeprom.h"
#include "pci76x.h"
#include "ioctrl.h"
#include "isr.h"
#include "version.h"

int ioctl_getporttype( unsigned long device, int port )
{
	switch(device)
	{
	case PCI762_16:
		if(port == 0)
			return DIOINOROUT;
		else if(port == 1)
			return DIOIN;
		else
			return -1;
	case PCI762_32:
		if(port == 0)
			return DIOINOROUT;
		else if( (port >= 1) || (port <= 2) )
			return DIOIN;
		else
			return -1;
	case PCI762_48:
		if(port == 0)
			return DIOINOROUT;
		else if( (port >= 1) || (port <= 3) )
			return DIOIN;
		else
			return -1;
	case PCI763_16:
		if( (port >= 0) || (port <= 1) )
			return DIOINOROUT;
		else
			return -1;
	case PCI763_32:
		if( (port >= 0) || (port <= 1) )
			return DIOINOROUT;
		else if(port == 2)
			return DIOOUT;
		else
			return -1;
	case PCI769:
		if( (port >= 0) || (port <= 1) )
			return DIOINOROUT;
		else
			return -1;
	}
	return -1;
}

int ioctl_query( pBoardDevice_t pdev,unsigned long arg )
{
	unsigned long param[2];
	copy_from_user(param,(void *)arg, 2 * sizeof(unsigned long));

	switch(param[0]) {
	case BRDSERIALNO:
		param[0] = pdev->devinfo.serialnumber;
		break;
	case BRDREV:
		param[0] = pdev->devinfo.revision;
		break;
	case BRDTYPE:
		param[0] = pdev->devinfo.boardtype;
		break;
	case BRDYEAR:
		param[0] = pdev->devinfo.year;
		break;
	case BRDMONTH:
		param[0] = pdev->devinfo.month;
		break;
	case BRDDAY:
		param[0] = pdev->devinfo.day;
		break;
	case BRDBUSTYPE:
		param[0] = PC_BUS_PCI;
		break;
	case DRVMAJOR:
		param[0] = (PCI76XDRV_VERSION >> 16) & 0xff;
		break;
	case DRVMINOR:
		param[0] = (PCI76XDRV_VERSION >> 8) & 0xff;
		break;
	case DRVBUILD:
		param[0] = PCI76XDRV_VERSION & 0xff;
		break;
	case ADNUMCHAN:
		param[0] = 0;
		break;
	case DANUMCHAN:
		param[0] = 0;
		break;
	case CTNUM:
		param[0] = 0;
		break;
	case DIONUMPORT:
		param[0] = pdev->devinfo.diochan;
		break;
	case DIOQRYPORT:
		param[0] = ioctl_getporttype(pdev->devinfo.boardtype, param[1]);
		break;
	case DIOPORTWIDTH:
		param[0] = pdev->devinfo.portwidth[param[1]];
		break;
	case INTNUMSRC:
		param[0] = pdev->devinfo.irqchan;
		break;
	case INTSTATUS:
		param[0] = pdev->irq_received;
		break;
	default:
		param[0]=-EINVAL;
		param[1]=0;
	}

	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}

int ioctl_diowrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,value;
	int ret;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	port = param[0];
	value = param[1];

	ret = ioctl_getporttype(pdev->devinfo.boardtype, port);
	if(ret < 0)
		return -EINVAL;
	if(ret == DIOIN)
		return -EINVAL;

	switch(port) {
	case 0:
		writel(param[1] & 0xffff, pdev->ptrmem + PCI76X_REG_DO_PORT);
		break;
	case 1:
		writel(param[1] & 0xffff, pdev->ptrmem + PCI76X_REG_RELAY_PORT);
		break;
	case 2:
		writel(param[1] & 0xffff, pdev->ptrmem + PCI76X_REG_RELAY1_PORT);
		break;
	}
	return EDRE_OK;
}

int ioctl_dioread( pBoardDevice_t pdev,unsigned long arg )
{
	unsigned long param[2],port,value;
	int ret;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port = param[0];

	ret = ioctl_getporttype(pdev->devinfo.boardtype, port);
	if(ret < 0)
		return -EINVAL;
	if(ret == DIOOUT)
		return -EINVAL;
	value = 0;

	switch(port) {
	case 0:
		value = readl(pdev->ptrmem + PCI76X_REG_DI_PORT);
		break;
	case 1:
		if( (pdev->devinfo.boardtype >= PCI763_16) &&
			(pdev->devinfo.boardtype<=PCI763_32) )
				value = 1 - (readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT));
		else
			value = 0xffff - readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT);
		break;
	case 2:
		value = 0xffff - readl(pdev->ptrmem + PCI76X_REG_OPTO1_PORT);
		break;
	case 3:
		value = 0xffff - readl(pdev->ptrmem + PCI76X_REG_OPTO2_PORT);
		break;
	}

	param[0] = value;
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}

int ioctl_intconfig( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[3],src,mode,type,tmp;
	copy_from_user(param, (void *)arg, 3 * sizeof(unsigned long));
	src = param[0];
	mode = param[1];
	type = param[2];

	if(pdev->devinfo.irqchan == 0)
		return -EIO;
	if(src >= pdev->devinfo.irqchan)
		return -EINVAL;
	if(mode >= 2)
		return -EINVAL;
	if(type >= 4)
		return -EINVAL;
	if(pdev->irq_status)
		return -EBUSY;

	irq_clear(pdev);

	if(src < 16) {
		if(mode == 1)
			pdev->irq_trigsrc0 |= mode << src;
		else if(mode == 0) {
			tmp = 0xffff - (1 << src);
			pdev->irq_trigsrc0 &= tmp;
		}
	}
	else if((src > 15) && (src < 32)) {
		if(mode == 1)
			pdev->irq_trigsrc1 |= mode << (src - 16);
		else if(mode == 0) {
			tmp = 0xffff - (1 << (src - 16));
			pdev->irq_trigsrc1 &= tmp;
		}
	}
	else if((src > 31) && (src < 48)) {
		if(mode == 1)
			pdev->irq_trigsrc2 |= mode << (src - 32);
		else if(mode == 0) {
			tmp = 0xffff - (1 << (src - 32));
			pdev->irq_trigsrc2 &= tmp;
		}
	}
	else if((src > 47) && (src < 64)) {
		if(mode == 1)
			pdev->irq_trigsrc3 |= mode << (src - 48);
		else if(mode == 0) {
			tmp = 0xffff - (1 << (src - 48));
			pdev->irq_trigsrc3 &= tmp;
		}
	}
	irq_settype(pdev, src, type);
	irq_clear(pdev);
	return EDRE_OK;
}

int ioctl_intenable( pBoardDevice_t pdev, unsigned long arg )
{
	irq_clear(pdev);
	return irq_enable(pdev);
}

int ioctl_intdisable( pBoardDevice_t pdev, unsigned long arg )
{
	irq_disable(pdev);
	return EDRE_OK;
}

int ioctl_intwait( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long flags;

	if(pdev->irq_wait > 0)
		return -EBUSY;

	spin_lock_irqsave(&pdev->lock,flags);
	pdev->irq_wait = 1;
	spin_unlock_irqrestore(&pdev->lock, flags);

	init_waitqueue_head(&pdev->wait_queue);
	wait_event_interruptible(pdev->wait_queue, 1);

	spin_lock_irqsave(&pdev->lock,flags);
	pdev->irq_wait = 0;
	spin_unlock_irqrestore(&pdev->lock,flags);

	return EDRE_OK;
}

int ioctl_intrelease( pBoardDevice_t pdev, unsigned long arg )
{
	if(pdev->irq_wait > 0)
		wake_up_interruptible(&pdev->wait_queue);
	return EDRE_OK;
}

/**
 * End of File
*/
