/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * isr.c -- Interrupt Service Routines source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "eeprom.h"
#include "pci76x.h"
#include "isr.h"
#include "version.h"

static irqreturn_t dev_isr( int irq, void *dev_id, struct pt_regs *reg )
{
	unsigned long status0, status1, status2, status3;
	pBoardDevice_t pdev = (pBoardDevice_t)dev_id;

	switch(pdev->devinfo.boardtype) {
	case PCI762_16:
		status1 = readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_STATUS) & 0xfff;
		status2 = 0;
		status3 = 0;
		break;
	case PCI762_32:
		status1 = readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_STATUS) & 0xfff;
		status2 = readl(pdev->ptrmem + PCI76X_REG_OPTO1_PORT_INT_STATUS) & 0xfff;
		status3 = 0;
		break;
	case PCI762_48:
		status1 = readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_STATUS) & 0xfff;
		status2 = readl(pdev->ptrmem + PCI76X_REG_OPTO1_PORT_INT_STATUS) & 0xfff;
		status3 = readl(pdev->ptrmem + PCI76X_REG_OPTO2_PORT_INT_STATUS) & 0xfff;
		break;
	case PCI763_16:
		status1 = readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_STATUS) & 0x1;
		status2 = 0;
		status3 = 0;
		break;
	case PCI763_32:
		status1 = readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_STATUS) & 0x1;
		status2 = 0;
		status3 = 0;
		break;
	case PCI769:
		status1 = readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_STATUS) & 0xfff;
		status2 = 0;
		status3 = 0;
		break;
	default :
		status1 = 0;
		status2 = 0;
		status3 = 0;
		break;
	}

	status0 = readl(pdev->ptrmem + PCI76X_REG_DI_PORT_INT_STATUS) & 0xfff;
	if(status0 > 0) {
		irq_devoff(pdev);
		irq_clear(pdev);
		pdev->irq_received = 0;
		pdev->irq_trigsrc = status0;
		wake_up_interruptible(&pdev->wait_queue);
		irq_devon(pdev);
	}
	if(status1 > 0) {
		irq_devoff(pdev);
		irq_clear(pdev);
		pdev->irq_received = 1;
		pdev->irq_trigsrc = status1;
		wake_up_interruptible(&pdev->wait_queue);
		irq_devon(pdev);
	}
	if(status2 > 0) {
		irq_devoff(pdev);
		irq_clear(pdev);
		pdev->irq_received = 2;
		pdev->irq_trigsrc = status2;
		wake_up_interruptible(&pdev->wait_queue);
		irq_devon(pdev);
	}
	if(status3>0) {
		irq_devoff(pdev);
		irq_clear(pdev);
		pdev->irq_received = 3;
		pdev->irq_trigsrc = status3;
		wake_up_interruptible(&pdev->wait_queue);
		irq_devon(pdev);
	}
	return IRQ_HANDLED;
}

void irq_init( pBoardDevice_t pdev )
{
	pdev->irq_status = IRQ_STATUS_IDLE;
	pdev->irq_received = 0;
	pdev->irq_cfg = 0;
	pdev->irq_type = 0;
}

int irq_enable( pBoardDevice_t pdev )
{
	if(pdev->irq_status == IRQ_STATUS_BUSY)
		return -EBUSY;

	if( request_irq( pdev->pci_device->irq,
		dev_isr,
		SA_SHIRQ,
		DEVICE,
		(void *)pdev) ) {
			err("%s: Failed to allocate irq." ,DEVICE);
			return -EIO;
		}

	irq_devon(pdev);
	pdev->irq_status=IRQ_STATUS_BUSY;
	return EDRE_OK;
}

void irq_disable( pBoardDevice_t pdev )
{
	irq_devoff(pdev);
	if(pdev->irq_status != IRQ_STATUS_BUSY)
		return;
	pdev->irq_status = IRQ_STATUS_IDLE;
	free_irq(pdev->pci_device->irq, pdev);
}

void irq_devon( pBoardDevice_t pdev )
{
	writel(pdev->irq_trigsrc0, pdev->ptrmem + PCI76X_REG_DI_PORT_INT_MASK);
	writel(pdev->irq_trigsrc1, pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_MASK);
	writel(pdev->irq_trigsrc2, pdev->ptrmem + PCI76X_REG_OPTO1_PORT_INT_MASK);
	writel(pdev->irq_trigsrc3, pdev->ptrmem + PCI76X_REG_OPTO2_PORT_INT_MASK);
}

void irq_devoff(pBoardDevice_t pdev)
{
	writel(0x0000, pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_MASK);
	writel(0x0000, pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_MASK);
	writel(0x0000, pdev->ptrmem + PCI76X_REG_OPTO1_PORT_INT_MASK);
	writel(0x0000, pdev->ptrmem + PCI76X_REG_OPTO2_PORT_INT_MASK);
}

void irq_settype( pBoardDevice_t pdev, unsigned int port, unsigned int type )
{
	unsigned long reg,tmp;

	if(port < 16) {
		reg = readl(pdev->ptrmem + PCI76X_REG_DI_PORT_INT_CFG);
		tmp = 0xffffffff - (0x3 << (port * 2));
		reg &= tmp;
		reg |= (type << (port * 2));
		writel(reg, pdev->ptrmem + PCI76X_REG_DI_PORT_INT_CFG);
	}
	else if((port > 15) && (port < 32)) {
		reg = readl(pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_CFG);
		tmp = 0xffffffff - (0x3 << ((port - 16) * 2));
		reg &= tmp;
		reg |= (type << ((port - 16) * 2));
		writel(reg, pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_CFG);
	}
	else if((port > 31) && (port < 48)) {
		reg = readl(pdev->ptrmem + PCI76X_REG_OPTO1_PORT_INT_CFG);
		tmp = 0xffffffff - (0x3 << ((port - 32) * 2));
		reg &= tmp;
		reg |= (type << ((port - 32) * 2));
		writel(reg, pdev->ptrmem + PCI76X_REG_OPTO1_PORT_INT_CFG);
	}
	else if((port > 47) && (port < 64)) {
		reg = readl(pdev->ptrmem + PCI76X_REG_OPTO2_PORT_INT_CFG);
		tmp = 0xffffffff-(0x3<<((port - 48) * 2));
		reg &= tmp;
		reg |= (type << ((port - 48) * 2));
		writel(reg, pdev->ptrmem + PCI76X_REG_OPTO2_PORT_INT_CFG);
	}
}

void irq_clear(pBoardDevice_t pdev)
{
	writel(0xffff, pdev->ptrmem + PCI76X_REG_DI_PORT_INT_STATUS);
	writel(0xffff, pdev->ptrmem + PCI76X_REG_OPTO_PORT_INT_STATUS);
	writel(0xffff, pdev->ptrmem + PCI76X_REG_OPTO1_PORT_INT_STATUS);
	writel(0xffff, pdev->ptrmem + PCI76X_REG_OPTO2_PORT_INT_STATUS);
	udelay(10);
}

/**
 * End of File
 */
