/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * ioctl.c -- I/O interface source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#include "stdinc.h"
#include "eeprom.h"
#include "pci773.h"
#include "version.h"
#include "ioctl.h"

extern void dev_readeeprom( pBoardDevice_t pdev );

int ioctl_query( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2];
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	switch(param[0]) {
	case BRDSERIALNO:
		param[0] = pdev->devinfo.serialnumber;
		break;
	case BRDREV:
		param[0] = pdev->devinfo.revision;
		break;
	case BRDTYPE:
		param[0] = pdev->devinfo.boardtype;
		break;
	case BRDYEAR:
		param[0] = pdev->devinfo.year;
		break;
	case BRDMONTH:
		param[0] = pdev->devinfo.month;
		break;
	case BRDDAY:
		param[0] = pdev->devinfo.day;
		break;
	case BRDBUSTYPE:
		param[0] = PC_BUS_PCI;
		break;
	case DRVMAJOR:
		param[0] = (PCI773DRV_VERSION>>16)&0xff;
		break;
	case DRVMINOR:
		param[0] = (PCI773DRV_VERSION>>8)&0xff;
		break;
	case DRVBUILD:
		param[0] = PCI773DRV_VERSION&0xff;
		break;
	case ADNUMCHAN:
		param[0] = pdev->devinfo.adcchan;
		break;
	case ADNUMTMP:
		param[0] = pdev->devinfo.tmpchan;
		break;
	case ADAMBCHAN:
		param[0] = pdev->devinfo.ambchan;
		break;
	case ADUNREAD:
		param[0] = 0;
		break;
	case ADMAXFREQ:
		param[0] = 0;
		break;
	case ADFIFOSIZE:
		param[0] = 0;
		break;
	case ADBUFFALLOC:
		param[0] = 0;
	case ADNUMRANGES:
		param[0] = 1;
		break;
	case ADRANGES:
		param[0] = BI_DIFF;
	case ADNUMMODES:
		param[0] = 1;
		break;
	case ADMODES:
		param[0] = BI_DIFF;
	case ADNUMGAINS:
		param[0] = 0;
		break;
	case ADGAINS:
		param[0] = 0;
		break;
	case ADNUMTRIG:
		param[0] = 0;
		break;
	case ADTRIG:
		param[0] = 0;
		break;
	case ADNUMTRIGMODE:
		param[0] = 0;
		break;
	case ADTRIGMODE:
		param[0] = 0;
		break;
	case ADNUMCLK:
		param[0] = 0;
		break;
	case ADCLK:
		param[0] = 0;
		break;
	case ADNUMGATES:
		param[0] = 0;
		break;
	case ADGATE:
		param[0] = 0;
		break;
	case DANUMCHAN:
		param[0] = 0;
		break;
	case CTNUM:
		param[0] = 0;
		break;
	case DIONUMPORT:
		param[0] = pdev->devinfo.diochan;
		break;
	case DIOQRYPORT:
		if(param[1])
			param[0] = DIOIN;
		else
			param[0] = DIOOUT;
		break;
	case DIOPORTWIDTH:
		param[0] = 16;
		break;
	case INTNUMSRC:
		param[0] = 0;
		break;
	case 1000:
		param[0] = ee_read(&pdev->eeprom, param[1]);
		break;
	case 1009:
		dev_readeeprom(pdev);
		param[0] = 0;
		break;
	case 1030:
	case 1031:
	case 1032:
	case 1033:
	case 1034:
	case 1035:
	case 1036:
	case 1037:
	case 1038:
	case 1039:
	case 1040:
	case 1041:
	case 1042:
	case 1043:
	case 1044:
	case 1045:
	case 1046:
	case 1047:
		pdev->caldata.offset[param[0] - 1030] = param[1];
		param[0] = 0;
		break;
	case 1050:
	case 1051:
	case 1052:
	case 1053:
	case 1054:
	case 1055:
	case 1056:
	case 1057:
	case 1058:
	case 1059:
	case 1060:
	case 1061:
	case 1062:
	case 1063:
	case 1064:
	case 1065:
	case 1066:
	case 1067:
		ee_write(&pdev->eeprom, param[0] - 1040, pdev->caldata.offset[param[0] - 1050]);
		param[0] = 0;
		break;
	case 1070:
	case 1071:
	case 1072:
	case 1073:
	case 1074:
	case 1075:
	case 1076:
	case 1077:
	case 1078:
	case 1079:
	case 1080:
	case 1081:
	case 1082:
	case 1083:
	case 1084:
	case 1085:
		pdev->caldata.gain[param[0] - 1070] = param[1];
		param[0] = 0;
		break;
	case 1090:
	case 1091:
	case 1092:
	case 1093:
	case 1094:
	case 1095:
	case 1096:
	case 1097:
	case 1098:
	case 1099:
	case 1100:
	case 1101:
	case 1102:
	case 1103:
	case 1104:
	case 1105:	
		ee_write(&pdev->eeprom, param[0] - 1060, pdev->caldata.gain[param[0] - 1090]);
		param[0] = 0;
		break;	
	default:
		param[0] = -1;
	}
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return 0;
}

int ioctl_diowrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2], port, value;

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	value = param[1];

	if(port != 0)
		return -EINVAL;

	writel(value & 0x0000ffff, pdev->ptrmem + 0x00000008);
	return 0;
}

int ioctl_dioread( pBoardDevice_t pdev, unsigned long arg)
{
	unsigned long param[2], port, value;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port = param[0];
	value = 0;

	if(port != 1)
		return -EINVAL;

	value = readl(pdev->ptrmem + 0x0000000cl);
	param[0] = value;
	copy_to_user((void *)arg, param,sizeof(unsigned long));
	return 0;
}

int ioctl_adsingle( pBoardDevice_t pdev, unsigned long arg )
{
	const unsigned long channels[18] = { 8,9,10,11,12,13,14,15,16,
										17,18,19,20,21,22,23,0,31 };
	const unsigned long gains[2] = { 0,0 };
	unsigned long param[2], val;
	unsigned char chan, gain;
	long uv, denom;

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	chan = param[0];
	gain = param[1];

	if(chan >= 18) 
		return -EINVAL;
	if(gain > 1) 
		return -EINVAL;

	if(chan >= 16)
		gain = 0;
	else
		gain = 1;

	val = ((gains[gain]<<5) | (channels[chan])) & 127;
	writel(val, pdev->ptrmem + 0x0l);
	udelay(20);

	val = readl(pdev->ptrmem + 0x04l);
	uv = ioctl_bintovoltage(val);

	if(gain == 1) {
		uv += (pdev->caldata.offset[chan + gain] - 10000);
		uv *= 10;
		denom = pdev->caldata.gain[chan];
		if(denom)			
			uv /= denom;
	}
	else {
		uv += (pdev->caldata.offset[gain] - 10000);
	}
	copy_to_user((void *)arg, &uv, sizeof(long));
	return 0;
}

int ioctl_adconfig( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2];

	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	pdev->range = (unsigned char)param[0];
	return 0;
}

long ioctl_bintovoltage( unsigned long bin )
{
	long uv;
	bin &= 0x3fff;
	if(bin < 8192) {
		uv = (bin * 1526) / 5;
	}
	else {
		bin &= 8191;
		uv = ((bin * 1526) / 5) - 2500000;
	}
	return uv;
}

/**
 * End of File
*/
