/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * ioctrl.c -- I/O Control source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "pci800inc.h"
#include "eeprom.h"
#include "pci800.h"
#include "ioctrl.h"
#include "isr.h"
#include "util.h"
#include "version.h"

int ioctl_query( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2];
	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));

	switch(param[0]) {
	case BRDSERIALNO:
		param[0] = pdev->devinfo.serialnumber;
		break;
	case BRDREV:
		param[0] = pdev->devinfo.revision;
		break;
	case BRDTYPE:
		param[0] = pdev->devinfo.boardtype;
		break;
	case BRDYEAR:
		param[0] = pdev->devinfo.year;
		break;
	case BRDMONTH:
		param[0] = pdev->devinfo.month;
		break;
	case BRDDAY:
		param[0] = pdev->devinfo.day;
		break;
	case BRDBUSTYPE:
		param[0] = PC_BUS_PCI;
		break;
	case DRVMAJOR:
		param[0] = (PCI800DRV_VERSION>>16)&0xff;
		break;
	case DRVMINOR:
		param[0] = (PCI800DRV_VERSION>>8)&0xff;
		break;
	case DRVBUILD:
		param[0] = PCI800DRV_VERSION&0xff;
		break;
	case ADNUMCHAN:
		param[0] = 0;
		break;
	case DANUMCHAN:
		param[0] = 0;
		break;
	case CTNUM:
		param[0] = pdev->devinfo.cntchan;
		break;
	case DIONUMPORT:
		param[0] = pdev->devinfo.diochan;
		break;
	case DIOQRYPORT:
		param[0] = utl_getporttype(pdev->devinfo.boardtype, param[1]);
		break;
	case DIOPORTWIDTH:
		param[0] = utl_getportwidth(pdev->devinfo.boardtype, param[1]);
		break;
	case INTNUMSRC:
		param[0] = pdev->devinfo.irqchan;
		break;
	case INTSTATUS:
		param[0] = pdev->irq_received;
		break;
	default:
		param[0] = 0;
	}
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}

int ioctl_diowrite( pBoardDevice_t pdev,unsigned long arg )
{
	unsigned long param[2],port,value,ppi;
	int ret;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	port = param[0];
	value = param[1];

	ret = utl_getporttype(pdev->devinfo.boardtype, port);
	if( (ret < 0) ||
		(ret == DIOIN) )
		return -EINVAL;

	if( (pdev->devinfo.hcport > 0) &&
		(port == pdev->devinfo.hcport) ) {
		writel(value, pdev->ptrmem + PCI800_CS1_OFFSET);
	}
	else {
		ppi = port / 3;
		port -= (ppi * 3);

		if(pdev->ppi[ppi].io[port] != DIOOUT) {
			pdev->ppi[ppi].io[port] = DIOOUT;
			utl_8255configure(pdev, ppi);
		}
		writel( param[1] & 0xff,
				pdev->ptrmem + PCI800_CS0_OFFSET * (ppi + 1) + port * 4);
	}
	return EDRE_OK;
}

int ioctl_dioread( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,value,ppi;
	int ret;

	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port=param[0];

	ret = utl_getporttype(pdev->devinfo.boardtype, port);
	if( (ret<0) ||
		(ret == DIOOUT) )
		return -EINVAL;

	if( (pdev->devinfo.optoport > 0) &&
		(port == pdev->devinfo.optoport) ) {
		value = readl(pdev->ptrmem + PCI800_REG_USR_INPUT) & 2;
		if(value > 0)
			value = 0;
		else
			value = 1;
	}
	else {
		ppi = port / 3;
		port -= (ppi * 3);

		if(pdev->ppi[ppi].io[port] != DIOIN) {
			pdev->ppi[ppi].io[port] = DIOIN;
			utl_8255configure(pdev, ppi);
		}
		value = readl( pdev->ptrmem +
						PCI800_CS0_OFFSET * (ppi + 1) + port * 4) & 0xff;
	}
	param[0] = value;
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}

int ioctl_ctwrite( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,value;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	port = param[0];
	value = param[1];

	if( (pdev->devinfo.cntchan == 0) ||
				(port > 3) )
		return -EINVAL;

	if(port < 3) {
		writel(value & 0xff, pdev->ptrmem + PCI800_CT0_OFFSET + (port * 4));
		writel( (value >> 8) & 0xff,
				 pdev->ptrmem + PCI800_CT0_OFFSET + (port * 4));
	}
	else {
		pdev->freqreg &= 0x0300;
		pdev->freqreg |= value&0xff;
		writel(pdev->freqreg, pdev->ptrmem + PCI800_REG_FREQ_DIV);
	}
	return EDRE_OK;
}

int ioctl_ctread( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,value;

	copy_from_user(param,(void *)arg, sizeof(unsigned long));
	port = param[0];

	if( (pdev->devinfo.cntchan == 0) ||
			(port > 3) )
		return -EINVAL;

	value = readl(pdev->ptrmem + PCI800_CT0_OFFSET + (port * 4)) & 0xff;
	value|=((readl(pdev->ptrmem + PCI800_CT0_OFFSET + (port * 4)) & 0xff) << 8);
	value &= 0xffff;
	param[0] = value;
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}


int ioctl_ctcfg( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[5],port,mode,bcd,clksrc,gatesrc;

	copy_from_user(param, (void *)arg, 5 * sizeof(unsigned long));
	port = param[0];
	mode = param[1];
	bcd = param[2];
	clksrc = param[3];
	gatesrc = param[4];     //not used

	if(pdev->devinfo.cntchan == 0)
		return -EIO;
	if(port > 3)
		return -EINVAL;

	if(port < 3)
		utl_8254configure(pdev, port, mode, bcd);
	else
		utl_writefreqregcfg(pdev, clksrc, mode);
	return EDRE_OK;
}

int ioctl_ctsoftgate( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[2],port,mode;

	copy_from_user(param, (void *)arg, 2 * sizeof(unsigned long));
	port = param[0];
	mode = param[1];

	if((port>2) || (mode>1))
		return -EINVAL;

	utl_writegate(pdev, port, mode);
	return EDRE_OK;
}

int ioctl_intconfig( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long param[3],src,mode,type;

	copy_from_user(param, (void *)arg, 3 * sizeof(unsigned long));
	src = param[0];
	mode = param[1];
	type = param[2];

	if(pdev->devinfo.irqchan == 0)
		return -EIO;
	if(src >= pdev->devinfo.irqchan)
		return -EINVAL;
	if(mode >= PCI800_INTMDE_NUM)
		return -EINVAL;
	if(type >= PCI800_INTTYPE_NUM)
		return -EINVAL;
	if(pdev->irq_status)
		return -EBUSY;

	//setup irq source
	pdev->irq_cfg &= (0x0000ffff - (1 << src));
	pdev->irq_cfg |= mode << src;
	writel(pdev->irq_cfg, pdev->ptrmem + PCI800_REG_INT_MASK);

	//setup mode
	pdev->irq_type &= 0x0000ffff - ( 1<< src);
	pdev->irq_type |= type << src;
	writel(pdev->irq_type, pdev->ptrmem + PCI800_REG_INT_CFG);

	//clear interrupt status
	writel(0x0000ffff, pdev->ptrmem + PCI800_REG_INT_STAT);
	return EDRE_OK;
}

int ioctl_intenable( pBoardDevice_t pdev, unsigned long arg )
{
	return irq_enable(pdev);
}

int ioctl_intdisable( pBoardDevice_t pdev, unsigned long arg )
{
	irq_disable(pdev);
	return EDRE_OK;
}

int ioctl_intwait( pBoardDevice_t pdev, unsigned long arg )
{
	unsigned long flags;

	if(pdev->irq_wait > 0)
		return -EBUSY;

	spin_lock_irqsave(&pdev->lock, flags);
	clear_bit(1, &pdev->irq_cond);
	spin_unlock_irqrestore(&pdev->lock, flags);

	init_waitqueue_head(&pdev->wait_queue);
	wait_event_interruptible(pdev->wait_queue, test_bit(1, &pdev->irq_cond));

	clear_bit(1, &pdev->irq_cond);
	spin_lock_irqsave(&pdev->lock, flags);
	pdev->irq_wait = 0;
	spin_unlock_irqrestore(&pdev->lock, flags);
	return EDRE_OK;
}

int ioctl_intrelease(pBoardDevice_t pdev, unsigned long arg)
{
	if(pdev->irq_wait > 0)
		wake_up_interruptible(&pdev->wait_queue);
	return EDRE_OK;
}

/**
 * End of File
*/
