/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * isr.c -- Interrupt Service Routine source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "pci800inc.h"
#include "version.h"
#include "eeprom.h"
#include "pci800.h"
#include "util.h"
#include "isr.h"

static irqreturn_t dev_isr( int irq, void *dev_id, struct pt_regs *reg )
{
	unsigned long val;
	pBoardDevice_t pdev = (pBoardDevice_t)dev_id;
	
	val = readl(pdev->ptrmem + PCI800_REG_INT_CORE) & PCI800_BIT_INT_STAT;
	if(!val)
		return -1;
	irq_updatestatus(pdev);
	irq_clear(pdev);
	set_bit(1, &pdev->irq_cond);
	wake_up_interruptible(&pdev->wait_queue);
	return IRQ_HANDLED;
}

void irq_init( pBoardDevice_t pdev )
{
	pdev->irq_status = IRQ_STATUS_IDLE;
	pdev->irq_received = 0;
	pdev->irq_cfg = 0;
	pdev->irq_type = 0;
	pdev->irq_wait = 0;
}

int irq_enable( pBoardDevice_t pdev )
{
	if(pdev->irq_status == IRQ_STATUS_BUSY)
		return -EBUSY;

	if( request_irq( pdev->pci_device->irq,
						dev_isr,
						SA_SHIRQ,
						DEVICE,
						(void *)pdev) ) {
		drv_err("%s: Failed to allocate irq." ,DEVICE);
		return -EIO;
	}

	irq_buson(pdev);
	pdev->irq_status = IRQ_STATUS_BUSY;
	return EDRE_OK;
}

void irq_disable( pBoardDevice_t pdev )
{
	irq_busoff(pdev);
	if(pdev->irq_status != IRQ_STATUS_BUSY)
		return;
	pdev->irq_status = IRQ_STATUS_IDLE;
	free_irq(pdev->pci_device->irq, pdev);
}

void irq_buson( pBoardDevice_t pdev )
{
	writel(0x00000100, pdev->ptrmem + PCI800_REG_INT_CORE);
	writel(0x00000300, pdev->ptrmem + PCI800_REG_INT_CORE);
}

void irq_busoff( pBoardDevice_t pdev )
{
	writel(0x00000100, pdev->ptrmem + PCI800_REG_INT_CORE);
}

void irq_updatestatus( pBoardDevice_t pdev )
{
	pdev->irq_trigsrc = readl(pdev->ptrmem + PCI800_REG_INT_STAT) & 0x3ff;
	writel(0x3ff, pdev->ptrmem + PCI800_REG_INT_STAT);
	pdev->irq_received++;
}

void irq_clear(pBoardDevice_t pdev)
{
	writel( PCI800_BIT_INT_STAT | PCI800_BIT_INT_ENABLE,
			pdev->ptrmem + PCI800_REG_INT_CORE );
}


/**
 * End of File
 */
