/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * pci800.c -- pci800 main source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/

#ifndef LINUX_VERSION_CODE
#  include <linux/version.h>
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
# error "This driver needs kernel 2.6.9 and later"
#endif
#include <linux/config.h>
#ifndef CONFIG_PCI
# error "This driver needs PCI support"
#endif
#include "stdinc.h"
#include "pci800inc.h"
#include "eeprom.h"
#include "pci800.h"
#include "ioctrl.h"
#include "util.h"
#include "isr.h"
#include "version.h"
#include "version.h"

#define DRIVER_AUTHOR "Jan Zwiegers, jan@eagle.co.za"
#define DRIVER_DESC "PCI800 Interface Driver (C) 2004-2005 Eagle Technology"

static int dev_read( struct file *file, char *buf, size_t length,
			loff_t * offset)
{
	return -EINVAL;
}

static int dev_write(struct file *file, const char *buf, size_t length,
					 loff_t * offset)
{
	return -EINVAL;
}

static void dev_kref_release( struct kref *kref )
{
	struct BoardDevice_t *dev = NULL;
	dev = container_of(kref, struct BoardDevice_t, kref);
	kfree(dev);
}

static int dev_open( struct inode *inode, struct file *pfile )
{
	pBoardDevice_t pdev;
	pdev = (pBoardDevice_t)pfile->private_data;
	spin_lock(&pdev->lock);
	spin_unlock(&pdev->lock);
	pfile->f_pos=0;
	kref_get(&pdev->kref);
	return 0;
}

static int dev_release( struct inode *inode, struct file *pfile )
{
	pBoardDevice_t pdev = (pBoardDevice_t)pfile->private_data;
	kref_put(&pdev->kref, dev_kref_release);
	return 0;
}

static int dev_ioctl( struct inode *inode, struct file *pfile,
					  unsigned int cmd, unsigned long arg )
{
	pBoardDevice_t pdev = (pBoardDevice_t)pfile->private_data;
	int eerr = 0;
	int size = _IOC_SIZE(cmd);
	if(_IOC_TYPE(cmd) != DEV_IOC_MAGIC)
		return -EINVAL;
	if(_IOC_NR(cmd) > DEV_IOC_MAXNR)
		return -EINVAL;
	if(_IOC_DIR(cmd) & _IOC_READ)
		eerr = verify_area(VERIFY_WRITE, (void *)arg, size);
	else if(_IOC_DIR(cmd) & _IOC_WRITE)
		eerr = verify_area(VERIFY_READ, (void *)arg, size);
	if(eerr)
		return eerr;

	switch(cmd) {
	case DEV_IOC_QUERY:
		eerr = ioctl_query(pdev, arg);
		break;
	case DEV_IOC_DIOWRITE:
		eerr = ioctl_diowrite(pdev, arg);
		break;
	case DEV_IOC_DIOREAD:
		eerr = ioctl_dioread(pdev, arg);
		break;
	case DEV_IOC_CTREAD:
		eerr = ioctl_ctread(pdev, arg);
		break;
	case DEV_IOC_CTWRITE:
		eerr = ioctl_ctwrite(pdev, arg);
		break;
	case DEV_IOC_CTCFG:
		eerr = ioctl_ctcfg(pdev, arg);
		break;
	case DEV_IOC_CTSOFTGATE:
		eerr = ioctl_ctsoftgate(pdev, arg);
		break;
	case DEV_IOC_INTCFG:
		eerr = ioctl_intconfig(pdev, arg);
		break;
	case DEV_IOC_INTEN:
		eerr = ioctl_intenable(pdev, arg);
		break;
	case DEV_IOC_INTDIS:
		eerr = ioctl_intdisable(pdev, arg);
		break;
	case DEV_IOC_INTWAIT:
		eerr = ioctl_intwait(pdev, arg);
		break;
	case DEV_IOC_INTRELEASE:
		eerr = ioctl_intrelease(pdev, arg);
		break;
	default:
		eerr = -EINVAL;
		break;
	}
	return eerr;
}

static struct file_operations device_fops = {
	.owner = THIS_MODULE,
	.open = dev_open,
	.release = dev_release,
	.ioctl = dev_ioctl,
	.read = dev_read,
	.write = dev_write,
};

void dev_inittype( pBoardDevice_t pdev )
{
	int n,i;
	switch(pdev->devinfo.boardtype) {
	case PCI836A:
		pdev->devinfo.cntchan = 0;
		pdev->devinfo.diochan = 3;
		pdev->devinfo.irqchan = 0;
		pdev->devinfo.hcport = 0;
		pdev->devinfo.optoport = 0;
		break;
	case PCI836C:
		pdev->devinfo.cntchan = 0;
		pdev->devinfo.diochan = 5;
		pdev->devinfo.irqchan = 3;
		pdev->devinfo.hcport = 3;
		pdev->devinfo.optoport = 4;
		break;
	case PCI848A:
		pdev->devinfo.cntchan = 0;
		pdev->devinfo.diochan = 6;
		pdev->devinfo.irqchan = 0;
		pdev->devinfo.hcport = 0;
		pdev->devinfo.optoport = 0;
		break;
	case PCI848C:
		pdev->devinfo.cntchan = 4;
		pdev->devinfo.diochan = 7;
		pdev->devinfo.irqchan = 10;
		pdev->devinfo.hcport = 0;
		pdev->devinfo.optoport = 6;
		break;
	case PCI896A:
		pdev->devinfo.cntchan = 0;
		pdev->devinfo.diochan = 12;
		pdev->devinfo.irqchan = 0;
		pdev->devinfo.hcport = 0;
		pdev->devinfo.optoport = 0;
		break;
	case PCI896C:
		pdev->devinfo.cntchan = 4;
		pdev->devinfo.diochan = 13;
		pdev->devinfo.irqchan = 10;
		pdev->devinfo.hcport = 0;
		pdev->devinfo.optoport = 12;
		break;
	case PCI8192A:
		pdev->devinfo.cntchan = 0;
		pdev->devinfo.diochan = 24;
		pdev->devinfo.irqchan = 0;
		pdev->devinfo.hcport = 0;
		pdev->devinfo.optoport = 0;
		break;
	case PCI8192C:
		pdev->devinfo.cntchan = 4;
		pdev->devinfo.diochan = 25;
		pdev->devinfo.irqchan = 10;
		pdev->devinfo.hcport = 0;
		pdev->devinfo.optoport = 24;
		break;
	default:
		pdev->devinfo.cntchan = 0;
		pdev->devinfo.diochan = 0;
		pdev->devinfo.irqchan = 0;
		pdev->devinfo.hcport = 0;
		pdev->devinfo.optoport = 0;
		break;
	}

	n = pdev->devinfo.diochan / 3;
	for(i=0;i<n;i++) {
		pdev->ppi[i].io[0] = 1;
		pdev->ppi[i].io[1] = 1;
		pdev->ppi[i].io[2] = 1;
		pdev->ppi[i].mode[0] = 0;
		pdev->ppi[i].mode[1] = 0;
		utl_8255configure(pdev,i);
	}
}

void dev_readeeprom( pBoardDevice_t pdev )
{
	pdev->devinfo.serialnumber = ee_read(&pdev->eeprom, 1);
	pdev->devinfo.serialnumber |= ee_read(&pdev->eeprom, 0) << 16;
	pdev->devinfo.year = ee_read(&pdev->eeprom, 2);
	pdev->devinfo.month = ee_read(&pdev->eeprom, 3) & 0xff;
	pdev->devinfo.day = (ee_read(&pdev->eeprom, 3) >> 8) & 0xff;
	pdev->devinfo.boardtype = ee_read(&pdev->eeprom, 4);
	pdev->devinfo.revision = ee_read(&pdev->eeprom, 5);
}

int dev_hwinit( pBoardDevice_t pdev )
{
	irq_init(pdev);
	ee_init(&pdev->eeprom, pdev->ptrmem + PCI800_REG_EEPROM);
	dev_readeeprom(pdev);

	if( (pdev->devinfo.serialnumber < 1000000001) ||
		(pdev->devinfo.serialnumber == 0xffffffff) ) {
		drv_err( "%s: Invalid Serial Number (%ld).",
			DEVICE, pdev->devinfo.serialnumber);
			return -ENODEV;
		}
	if( (pdev->devinfo.boardtype < PCI836A) ||
		(pdev->devinfo.boardtype>PCI8192C)) {
		drv_err( "%s: Invalid Board Type (%d).",
			 DEVICE, pdev->devinfo.boardtype);
		return -ENODEV;

	}

	drv_info( "%s: Added new device (serial number #%ld, type #%d).",
		  DEVICE, pdev->devinfo.serialnumber, pdev->devinfo.boardtype);

	dev_inittype(pdev);
	writel(0xc2, pdev->ptrmem + PCI800_REG_USR_CFG);
	writel(5, pdev->ptrmem + PCI800_REG_USR_OUTPUT);
	if(pdev->devinfo.boardtype == PCI836C)
		writel(8, pdev->ptrmem + PCI800_REG_USR_OUTPUT);
	else
		writel(8, pdev->ptrmem + PCI800_REG_USR_OUTPUT);
	writel(0x1ff, pdev->ptrmem + PCI800_REG_CORE_CFG);
	writel(0, pdev->ptrmem + PCI800_REG_INT_MASK);
	writel(0x02ff, pdev->ptrmem + PCI800_REG_INT_STAT);
	return 0;
}

void dev_hwuninit( pBoardDevice_t pdev )
{
  irq_disable(pdev);
  writel(255, pdev->ptrmem + PCI800_REG_USR_CFG);
}

int dev_alloc( struct pci_dev *pcidev, pBoardDevice_t pdev )
{
  spin_lock_init(&pdev->lock);

  pdev->pci_device = pcidev;
  pdev->ptrmem = ioremap_nocache( pci_resource_start(pcidev, 0),
								   pci_resource_len(pcidev, 0));
  if(!pdev->ptrmem) {
	  spin_unlock(&pdev->lock);
	  drv_err("%s: Unable to remap BAR.", DEVICE);
	  return -ENOMEM;
  }

  if(dev_hwinit(pdev)) {
	  spin_unlock(&pdev->lock);
	  drv_err("%s: Failed to initialize hardware.", DEVICE);
	  return -ENODEV;
  }
  return 0;
}

void dev_dealloc( pBoardDevice_t pdev )
{
	dev_hwuninit(pdev);
	iounmap(pdev->ptrmem);
}

static ssize_t dev_show_serial( struct device *device, char *buf )
{
	char s[20];
	pBoardDevice_t pbd = (pBoardDevice_t)device->driver_data;
	sprintf(s,"%ld\n", pbd->devinfo.serialnumber);
	strcpy(buf, s);
	return strlen(s) + 1;
}
static DEVICE_ATTR(serial, S_IRUGO, dev_show_serial, NULL);

static int __devinit dev_probe( struct pci_dev *dev,
				const struct pci_device_id *id )
{
	pBoardDevice_t pbd;

	pbd = kmalloc(sizeof(*pbd), GFP_KERNEL);
	if(!pbd) {
		drv_err("%s: Failed to allocate memory.", DEVICE);
		return -ENOMEM;
	}
	if(pci_enable_device(dev)) {
		drv_err("%s: Failed to enable PCI device.", DEVICE);
		kfree(pbd);
		return -ENODEV;
	}
	if(pci_request_regions(dev, DEVICE)) {
		drv_err("%s: Failed to request PCI regions.", DEVICE);
		kfree(pbd);
		return -ENODEV;
	}
	if(dev_alloc(dev, pbd))	{
		drv_err("%s: Failed to setup device.", DEVICE);
		kfree(pbd);
		pci_release_regions(dev);
		return -ENODEV;
	}
	pci_set_drvdata(dev, pbd);
	device_create_file(&dev->dev, &dev_attr_serial);
	pbd->minor = edredaq_register_dev( &dev->dev,
					&device_fops,
					(void *)pbd );
	kref_init(&pbd->kref);
	return 0;
}

static void __devexit dev_remove( struct pci_dev *dev )
{
	pBoardDevice_t pbd = (pBoardDevice_t)pci_get_drvdata(dev);
	dev_dealloc(pbd);
	edredaq_deregister_dev(pbd->minor);
	device_remove_file(&dev->dev, &dev_attr_serial);
	pci_release_regions(dev);
	pci_disable_device(dev);
	pci_set_drvdata(dev,NULL);
}

static int dev_suspend( struct pci_dev *dev, u32 state )
{
	return -ENODEV;
}

static int dev_resume( struct pci_dev *dev )
{
	return -ENODEV;
}

static int dev_enable_wake( struct pci_dev *dev, u32 state, int enable )
{
	return -ENODEV;
}

static struct pci_device_id pci_ids_tbl[] = {
	{ vendor: 0xea01l, device: 0x0800l,
	subvendor: PCI_ANY_ID, subdevice: PCI_ANY_ID,
	class: 0, class_mask: 0, driver_data: 0},
	{ 0, }
};
MODULE_DEVICE_TABLE(pci,pci_ids_tbl);

static struct pci_driver pci_drv = {
	.name = DEVICE,
	.probe = dev_probe,
	.remove = __devexit_p(dev_remove),
	.suspend = dev_suspend,
	.resume = dev_resume,
	.enable_wake = dev_enable_wake,
	.id_table = pci_ids_tbl,
};

static ssize_t drv_show_version( struct device_driver *drv, char *buf )
{
	strcpy(buf, DRIVER_VERSION);
	return strlen(DRIVER_VERSION) + 1;
}

static DRIVER_ATTR(version, S_IRUGO, drv_show_version, NULL);

static int __init start_module(void)
{
	drv_info("%s (%s)", DRIVER_DESC, DRIVER_VERSION);
	if(pci_register_driver(&pci_drv)<0) {
		drv_err("%s: Failed to register driver with PCI sub-system.", DEVICE);
		return -ENODEV;
	}
	driver_create_file(&pci_drv.driver, &driver_attr_version);
	return EDRE_OK;
}

static void __exit stop_module(void)
{
	drv_info("%s: unloading...", DEVICE);
	driver_remove_file(&pci_drv.driver, &driver_attr_version);
	pci_unregister_driver(&pci_drv);
}


module_init(start_module);
module_exit(stop_module);
MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL");
/**
 * End of File
 */

