/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * util.c -- Utility functions source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "pci800inc.h"
#include "eeprom.h"
#include "pci800.h"
#include "util.h"

void utl_8255configure( pBoardDevice_t pdev, int port )
{
	unsigned char cfg;
	unsigned char cval[4] = {0,9,8,1};
	cfg = (1<<7) |
			(pdev->ppi[port].mode[0] << 5) |
			(pdev->ppi[port].io[0] << 4) |
			(pdev->ppi[port].mode[1] << 2) |
			(pdev->ppi[port].io[1] << 1) |
			(cval[pdev->ppi[port].io[2]] );
	writel(cfg, pdev->ptrmem + PCI800_CS0_OFFSET + 12 * (port + 1));
}

int utl_getporttype( unsigned long device, int port )
{
	switch(device) {
	case PCI836A:
		if(port > 2)
			return -1;
		return DIOINOROUT;
	case PCI836C:
		if(port > 4)
			return -1;
		if(port < 3)
			return DIOINOROUT;
		if(port == 3)
			return DIOOUT;
		if(port == 4)
			return DIOIN;
		return -1;
	case PCI848A:
		if(port > 5)
			return -1;
		return DIOINOROUT;
	case PCI848C:
		if(port > 6)
			return -1;
		if(port < 6)
			return DIOINOROUT;
		else
			return DIOIN;
	case PCI896A:
		if(port > 11)
			return -1;
		return DIOINOROUT;
	case PCI896C:
		if(port > 12)
			return -1;
		if(port < 12)
			return DIOINOROUT;
		else
			return DIOIN;
	case PCI8192A:
		if(port > 23)
			return -1;
		return DIOINOROUT;
	case PCI8192C:
		if(port > 24)
			return -1;
		if(port < 24)
			return DIOINOROUT;
		else
			return DIOIN;
	}
	return -1;
}

int utl_getportwidth( unsigned long device, int port )
{
	switch(device) {
	case PCI836A:
		if(port > 2)
			return -1;
		return 8l;
	case PCI836C:
		if(port > 4)
			return -1;
		if(port < 3)
			return 8l;
		if(port == 3)
			return 16l;
		if(port == 4)
			return 1l;
	case PCI848A:
		if(port > 5)
			return -1;
		return 8l;
	case PCI848C:
		if(port > 6)
			return -1;
		if(port < 6)
			return 8l;
		else
			return 1l;
	case PCI896A:
		if(port > 11)
			return -1;
		return 8l;
	case PCI896C:
		if(port > 12)
			return -1;
		if(port < 12)
			return 8l;
		else
			return 1l;
	case PCI8192A:
		if(port > 23)
			return -1;
		return 8l;
	case PCI8192C:
		if(port > 24)
			return -1;
		if(port < 24)
			return 8l;
		else
			return 1l;
	}
	return 0;
}

void utl_8254configure( pBoardDevice_t pdev, int ct, int mode, int bcd )
{
	unsigned char cfg;
	if(mode > 5)
		mode = 5;
	if(ct > 2)
		ct = 2;
	if(bcd > 1)
		bcd=1;
	cfg = (ct<<6) | (CMD_RW_LSBMSB<<4) | (mode<<1) | bcd;
	writel(cfg, pdev->ptrmem + PCI800_CT0_OFFSET + 12);
}

void utl_writefreqregcfg( pBoardDevice_t pdev, int clksrc, int mode )
{
  if(clksrc > 1)
	  clksrc = 1;
  if(mode > 1)
	  mode=1;
  pdev->freqreg &= 0x0ff;
  pdev->freqreg |= (clksrc << 9) | (mode << 8);
  writel(pdev->freqreg, pdev->ptrmem + PCI800_REG_FREQ_DIV);
}

void utl_writegate( pBoardDevice_t pdev, int port, int mode )
{
	if(port > 2)
		port = 2;
	if(mode > 1)
		mode = 2;
	pdev->usrreg &= (0x00ff - ( 1<< (3 + port)));
	pdev->usrreg |= mode << (3 + port);
	writel(pdev->usrreg, pdev->ptrmem + PCI800_REG_USR_OUTPUT);
}

/**
 * End of File
*/
