/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * deviceio.c -- Device I/O source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "udaqlite.h"
#include "usbio.h"
#include "buffer.h"
#include "ioctl.h"
#include "deviceio.h"

void usbupc_eeinfo_dump( pEepromInfo_t ee )
{
#ifdef DEBUG
	err("usbupc: usbupc_eeinfo_dump.");
	err("usbupc: serialnumber: %ld", ee->serialnumber);
	err("usbupc: boardtype: %d", ee->boardtype);
	err("usbupc: revison: %d", ee->revision);
	err("usbupc: man date: %d-%d-%d", ee->day, ee->month, ee->year);
	err("usbupc: firmware version: %d.%d.%d",
	ee->firmmajor,ee->firmminor,ee->firmbuild);
#endif /*DEBUG*/
}

int usbupc_read_eeprom( pUSBuPcDevice_t dev, unsigned short position )
{
	unsigned char buf[64];
	buf[0] = USB_EEREAD>>8;
	buf[1] = USB_EEREAD;
	buf[2] = position>>8;
	buf[3] = position;
	usbupc_bulkpacket(dev, buf, 64);
	if(buf[1] != 0)
		return -EIO;
	return (int)buf[5];	
}

int usbupc_query_device( pUSBuPcDevice_t dev, long code, long param,
			 unsigned long *value )
{
	unsigned char buf[64];
	unsigned long val;
	buf[0] = USB_QUERY>>8;
	buf[1] = USB_QUERY;
	buf[2] = code>>8;
	buf[3] = code;
	buf[4] = param>>8;
	buf[5] = param;
	usbupc_bulkpacket(dev, buf, 64);
	if(buf[1] != 0)
		return -EIO;
	val = 0;
	val = buf[2]<<24 | buf[3]<<16 | buf[4]<<8 | buf[5];
	*value = val;
	return EOK;
}

int usbupc_get_adcconfig( pUSBuPcDevice_t dev )
{
	unsigned char buf[64];
	unsigned short qry = 0x1000;
	int i;
	
	buf[0] = qry>>8;
	buf[1] = qry;
	usbupc_bulkpacket(dev, buf, 64);
	
	if(buf[1] != 0)
		return -EIO;
	
	for(i=0; i<12; i++) {
		dev->adc.offset[i] = buf[2 + i];
	}
	
	dev->adc.gain_error = buf[14];
	return EOK;
}

int usbupc_get_dacconfig( pUSBuPcDevice_t dev )
{
	unsigned char buf[64];
	unsigned short qry = 0x1001;
	
	buf[0] = qry>>8;
	buf[1] = qry;
	usbupc_bulkpacket(dev, buf, 64);
	
	if(buf[1] != 0)
		return -EIO;
	
	dev->eeinfo.dac_offset[0] = buf[2];
	dev->eeinfo.dac_offset[1] = buf[3];
	dev->eeinfo.dac_gain_error[0] = buf[4];
	dev->eeinfo.dac_gain_error[1] = buf[5];
	
	return EOK;}

void usbupc_set_devicetype( pUSBuPcDevice_t dev )
{	
	pEepromInfo_t eeinfo=&dev->eeinfo;

	eeinfo->diochan = 2;
	eeinfo->cntchan = 1;
	eeinfo->adcchan = 8;
	eeinfo->dacchan = 2;
	
	eeinfo->adc_freq = 49000;
	eeinfo->adc_numranges = 8;
	eeinfo->adc_ranges[0] = 40000;
	eeinfo->adc_ranges[1] = 20000;
	eeinfo->adc_ranges[2] = 10000;
	eeinfo->adc_ranges[3] = 8000;
	eeinfo->adc_ranges[4] = 5000;
	eeinfo->adc_ranges[5] = 4000;
	eeinfo->adc_ranges[6] = 2500;
	eeinfo->adc_ranges[7] = 2000;
	
	eeinfo->adc_nummodes = 2;
	eeinfo->adc_modes[0] = BI_SINGLE;
	eeinfo->adc_modes[1] = BI_DIFF;

	eeinfo->adc_numtrig = 2;
	eeinfo->adc_trig[0] = AD_TRIG_INTERNAL;
	eeinfo->adc_trig[1] = AD_TRIG_EXTERNAL;
	
	eeinfo->adc_numtrigmode = 1;
	eeinfo->adc_trigmode[0] = AD_TRIG_POS;
	
	eeinfo->adc_numclk = 2;
	eeinfo->adc_clk[0] = AD_CLK_INTERNAL;
	eeinfo->adc_clk[1] = AD_CLK_EXTERNAL;
	
	eeinfo->dac_freq = 1;
	eeinfo->dac_speed = 1;
	eeinfo->dac_power = 0;

}

int usbupc_get_deviceinfo( pUSBuPcDevice_t dev )
{
	unsigned long vu32;
	if(usbupc_query_device(dev, BRDSERIALNO, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.serialnumber = vu32;
	if(usbupc_query_device(dev, BRDTYPE, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.boardtype = 0x2100;
	if(usbupc_query_device(dev, BRDREV, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.revision = vu32;
	if(usbupc_query_device(dev, BRDDAY, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.day = vu32;
	if(usbupc_query_device(dev, BRDMONTH, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.month = vu32;
	if(usbupc_query_device(dev, BRDYEAR, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.year = vu32;
	if(usbupc_query_device(dev, FRMMAJOR, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.firmmajor = vu32;
	if(usbupc_query_device(dev, FRMMINOR, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.firmminor = vu32;
	if(usbupc_query_device(dev, FRMBUILD, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.firmbuild = vu32;
	
	usbupc_get_adcconfig(dev);
	usbupc_get_dacconfig(dev);
	
	usbupc_eeinfo_dump(&dev->eeinfo);

	return EOK;
}

int usbupc_init_device( pUSBuPcDevice_t dev )
{
	int i;
	dev->adc.dblbuffer.data = kmalloc(DOUBLE_BUFFER_SIZE, GFP_KERNEL);
	if(dev->adc.dblbuffer.data == NULL)
		dev->adc.dblbuffer.size=0;
	else
		dev->adc.dblbuffer.size = DOUBLE_BUFFER_SIZE;
	usbupc_buf_init(&dev->adc.buffer, ADC_BUFFER_SIZE);
	for(i=0; i<ISO_NUM_URB; i++)
		dev->adc.isourb[i].pbuffer = &dev->adc.buffer;
	dev->iso_context.busy = 0;
	dev->adc.status = 0;
	return EOK;
}

void usbupc_uninit_device( pUSBuPcDevice_t dev )
{
	usbupc_isoxfr_stop(dev);
	usbupc_buf_uninit(&dev->adc.buffer);
	if(dev->adc.dblbuffer.size > 0) kfree(dev->adc.dblbuffer.data);
}

/**
 * End of File
 */
