/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * deviceio.c -- Device I/O source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "usbupc.h"
#include "usbio.h"
#include "buffer.h"
#include "ioctl.h"
#include "deviceio.h"

void usbupc_eeinfo_dump( pEepromInfo_t ee )
{
#ifdef DEBUG
	err("usbupc: usbupc_eeinfo_dump.");
	err("usbupc: serialnumber: %ld", ee->serialnumber);
	err("usbupc: boardtype: %d", ee->boardtype);
	err("usbupc: revison: %d", ee->revision);
	err("usbupc: man date: %d-%d-%d", ee->day, ee->month, ee->year);
	err("usbupc: firmware version: %d.%d.%d",
	ee->firmmajor,ee->firmminor,ee->firmbuild);
#endif /*DEBUG*/
}

void usbupc_dio_init( pUSBuPcDevice_t dev )
{
	unsigned char buf[64];
	unsigned char i;
	pEepromInfo_t eeinfo=&dev->eeinfo;
	
	for(i=0; i<eeinfo->diochan; i++) {
		switch(eeinfo->dio_type[i]) {
		case DIOIN:
			buf[0] = USB_DIOREAD>>8;
			buf[1] = USB_DIOREAD;
			buf[2] = eeinfo->dio_route[i]>>8;
			buf[3] = eeinfo->dio_route[i];
			usbupc_bulkpacket(dev, buf, 64);
			break;
		case DIOOUT:
			buf[0] = USB_DIOWRITE>>8;
			buf[1] = USB_DIOWRITE;
			buf[2] = eeinfo->dio_route[i]>>8;
			buf[3] = eeinfo->dio_route[i];
			buf[4] = 0;
			buf[5] = 0;
			usbupc_bulkpacket(dev, buf, 64);
			break;
		}
	}
}

int usbupc_query_device( pUSBuPcDevice_t dev, long code, long param,
			 unsigned long *value )
{
	unsigned char buf[64];
	unsigned long val;
	buf[0] = USB_QUERY>>8;
	buf[1] = USB_QUERY;
	buf[2] = code>>8;
	buf[3] = code;
	buf[4] = param>>8;
	buf[5] = param;
	usbupc_bulkpacket(dev, buf, 64);
	if(buf[1] != 0)
		return -EIO;
	val = 0;
	val = buf[2]<<24 | buf[3]<<16 | buf[4]<<8 | buf[5];
	*value = val;
	return EOK;
}

void usbupc_set_devicetype( pUSBuPcDevice_t dev )
{
	unsigned long vu32;
	pEepromInfo_t eeinfo=&dev->eeinfo;

	eeinfo->diochan = 0;
	eeinfo->cntchan = 0;
	eeinfo->adcchan = 0;
	eeinfo->dacchan = 0;
	eeinfo->tmpchan = 0;

	switch(eeinfo->boardtype)
	{
	case USB24A:
		eeinfo->diochan=3;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB24C:
		eeinfo->diochan=3;
		eeinfo->cntchan=6;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB48A:
		eeinfo->diochan=6;	
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 5;
		break;
	case USB48C:
		eeinfo->diochan=6;
		eeinfo->cntchan=6;
		eeinfo->irqchan=6;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 27;
		eeinfo->dio_route[4] = 28;
		eeinfo->dio_route[5] = 29;
		break;
	case USB72A:
		eeinfo->diochan=9;	
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 5;
		eeinfo->dio_route[6] = 6;
		eeinfo->dio_route[7] = 7;
		eeinfo->dio_route[8] = 8;
		break;
	case USB72C:
		eeinfo->diochan=9;
		eeinfo->cntchan=6;	
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 5;
		eeinfo->dio_route[6] = 27;
		eeinfo->dio_route[7] = 28;
		eeinfo->dio_route[8] = 29;
		break;
	case USB96A:
		eeinfo->diochan=12;	
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 5;
		eeinfo->dio_route[6] = 6;
		eeinfo->dio_route[7] = 7;
		eeinfo->dio_route[8] = 8;
		eeinfo->dio_route[9] = 9;
		eeinfo->dio_route[10] = 10;
		eeinfo->dio_route[11] = 11;
		break;
	case USB96C:
		eeinfo->diochan=12;
		eeinfo->cntchan=6;
		eeinfo->irqchan=6;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 5;
		eeinfo->dio_route[6] = 6;
		eeinfo->dio_route[7] = 7;
		eeinfo->dio_route[8] = 8;
		eeinfo->dio_route[9] = 27;
		eeinfo->dio_route[10] = 28;
		eeinfo->dio_route[11] = 29;
		break;
	case USB120A:
		eeinfo->diochan=15;		
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 5;
		eeinfo->dio_route[6] = 6;
		eeinfo->dio_route[7] = 7;
		eeinfo->dio_route[8] = 8;
		eeinfo->dio_route[9] = 9;
		eeinfo->dio_route[10] = 10;
		eeinfo->dio_route[11] = 11;
		eeinfo->dio_route[12] = 12;
		eeinfo->dio_route[13] = 13;
		eeinfo->dio_route[14] = 14;
		break;
	case USB120C:
		eeinfo->diochan=15;
		eeinfo->cntchan=6;		
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 5;
		eeinfo->dio_route[6] = 6;
		eeinfo->dio_route[7] = 7;
		eeinfo->dio_route[8] = 8;
		eeinfo->dio_route[9] = 9;
		eeinfo->dio_route[10] = 10;
		eeinfo->dio_route[11] = 11;
		eeinfo->dio_route[12] = 27;
		eeinfo->dio_route[13] = 28;
		eeinfo->dio_route[14] = 29;
		break;
	case USB73T8:
		eeinfo->diochan=3;
		eeinfo->adcchan=9;
		eeinfo->tmpchan=9;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB73T16:
		eeinfo->diochan=3;
		eeinfo->adcchan=18;
		eeinfo->tmpchan=18;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB73T32:
		eeinfo->diochan=3;
		eeinfo->adcchan=36;
		eeinfo->tmpchan=36;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB73R8:
		eeinfo->diochan=3;
		eeinfo->adcchan=9;
		eeinfo->tmpchan=9;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB73R16:
		eeinfo->diochan=3;
		eeinfo->adcchan=18;
		eeinfo->tmpchan=18;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB73R32:
		eeinfo->diochan=3;
		eeinfo->adcchan=36;
		eeinfo->tmpchan=36;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB26A16:
	case USB26B16:
	case USB26C16:
	case USB26D16:
		eeinfo->diochan=3;
		eeinfo->adcchan=16;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB26A32:
	case USB26B32:
	case USB26C32:
	case USB26D32:
		eeinfo->diochan=3;
		eeinfo->adcchan=32;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB30A16:
	case USB30B16:
	case USB30C16:
	case USB30D16:
		eeinfo->diochan=3;
		eeinfo->adcchan=16;
		eeinfo->dacchan=4;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB30A32:
	case USB30B32:
		eeinfo->diochan=3;
		eeinfo->adcchan=32;
		eeinfo->dacchan=8;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB30C32:
	case USB30D32:
		eeinfo->diochan=3;
		eeinfo->adcchan=32;
		eeinfo->dacchan=4;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		break;
	case USB62_16:
		eeinfo->diochan=5;
		eeinfo->dio_type[0] = DIOINOROUT;
		eeinfo->dio_type[1] = DIOINOROUT;
		eeinfo->dio_type[2] = DIOINOROUT;
		eeinfo->dio_type[3] = DIOIN;
		eeinfo->dio_type[4] = DIOIN;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		usbupc_dio_init(dev);
		break;
	case USB62_32:
		eeinfo->diochan=7;
		eeinfo->dio_type[0] = DIOINOROUT;
		eeinfo->dio_type[1] = DIOINOROUT;
		eeinfo->dio_type[2] = DIOINOROUT;
		eeinfo->dio_type[3] = DIOIN;
		eeinfo->dio_type[4] = DIOIN;
		eeinfo->dio_type[5] = DIOIN;
		eeinfo->dio_type[6] = DIOIN;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 9;
		eeinfo->dio_route[6] = 10;
		usbupc_dio_init(dev);
		break;
	case USB63_16:
		eeinfo->diochan=5;
		eeinfo->dio_type[0] = DIOINOROUT;
		eeinfo->dio_type[1] = DIOINOROUT;
		eeinfo->dio_type[2] = DIOINOROUT;
		eeinfo->dio_type[3] = DIOOUT;
		eeinfo->dio_type[4] = DIOOUT;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		usbupc_dio_init(dev);
		break;
	case USB63_32:
		eeinfo->diochan=7;
		eeinfo->dio_type[0] = DIOINOROUT;
		eeinfo->dio_type[1] = DIOINOROUT;
		eeinfo->dio_type[2] = DIOINOROUT;
		eeinfo->dio_type[3] = DIOOUT;
		eeinfo->dio_type[4] = DIOOUT;
		eeinfo->dio_type[5] = DIOOUT;
		eeinfo->dio_type[6] = DIOOUT;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 9;
		eeinfo->dio_route[6] = 10;
		usbupc_dio_init(dev);
		break;
	case USB69_16:
		eeinfo->diochan=5;
		eeinfo->dio_type[0] = DIOINOROUT;
		eeinfo->dio_type[1] = DIOINOROUT;
		eeinfo->dio_type[2] = DIOINOROUT;
		eeinfo->dio_type[3] = DIOOUT;
		eeinfo->dio_type[4] = DIOIN;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		usbupc_dio_init(dev);
		break;
	case USB69_32:
		eeinfo->diochan=7;
		eeinfo->dio_type[0] = DIOINOROUT;
		eeinfo->dio_type[1] = DIOINOROUT;
		eeinfo->dio_type[2] = DIOINOROUT;
		eeinfo->dio_type[3] = DIOOUT;
		eeinfo->dio_type[4] = DIOIN;
		eeinfo->dio_type[5] = DIOOUT;
		eeinfo->dio_type[6] = DIOIN;
		eeinfo->dio_route[0] = 0;
		eeinfo->dio_route[1] = 1;
		eeinfo->dio_route[2] = 2;
		eeinfo->dio_route[3] = 3;
		eeinfo->dio_route[4] = 4;
		eeinfo->dio_route[5] = 9;
		eeinfo->dio_route[6] = 10;
		usbupc_dio_init(dev);
		break;
	case USBRUGDAQ:
		usbupc_query_device(dev, 2100, 50, &vu32);
		eeinfo->diochan = vu32;
		usbupc_query_device(dev, 2100, 82, &vu32);
		eeinfo->adcchan = vu32;
		usbupc_query_device(dev, 2100, 83, &vu32);
		eeinfo->dacchan = vu32;
		usbupc_query_device(dev, 2100, 84, &vu32);
		eeinfo->tmpchan = vu32;
		usbupc_query_device(dev, 2100, 81, &vu32);
		eeinfo->cntchan = vu32;
		usbupc_query_device(dev, 2100, 85, &vu32);
		eeinfo->irqchan = vu32;
		break;
	}	
	

	//set analog properties
	switch(eeinfo->boardtype) {
	case USB26A16:
	case USB26A32:
	case USB30A16:
	case USB30A32:
	case USB26B16:
	case USB26B32:
	case USB30B16:
	case USB30B32:
		eeinfo->adc_devtype = 0;
		eeinfo->adc_numranges = 9;
		eeinfo->adc_ranges[0] = 20000;
		eeinfo->adc_ranges[1] = 10000;
		eeinfo->adc_ranges[2] = 5000;
		eeinfo->adc_ranges[3] = 2000;
		eeinfo->adc_ranges[4] = 1000;
		eeinfo->adc_ranges[5] = 500;
		eeinfo->adc_ranges[6] = 200;
		eeinfo->adc_ranges[7] = 100;
		eeinfo->adc_ranges[8] = 50;
		eeinfo->adc_nummodes = 4;
		eeinfo->adc_modes[0] = UNI_SINGLE;
		eeinfo->adc_modes[1] = BI_SINGLE;
		eeinfo->adc_modes[2] = UNI_DIFF;
		eeinfo->adc_modes[3] = BI_DIFF;
		eeinfo->adc_numtrig = 2;
		eeinfo->adc_trig[0] = AD_TRIG_INTERNAL;
		eeinfo->adc_trig[1] = AD_TRIG_EXTERNAL;
		eeinfo->adc_numtrigmode = 1;
		eeinfo->adc_trigmode[0] = AD_TRIG_POS;
		eeinfo->adc_numclk = 2;
		eeinfo->adc_clk[0] = AD_CLK_INTERNAL;
		eeinfo->adc_clk[1] = AD_CLK_EXTERNAL;
	}

}

int usbupc_get_deviceinfo( pUSBuPcDevice_t dev )
{
	unsigned long vu32,i;
	unsigned short vu16;
	if(usbupc_query_device(dev, BRDSERIALNO, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.serialnumber = vu32;
	if(usbupc_query_device(dev, BRDTYPE, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.boardtype = vu32;
	if(usbupc_query_device(dev, BRDREV, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.revision = vu32;
	if(usbupc_query_device(dev, BRDDAY, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.day = vu32;
	if(usbupc_query_device(dev, BRDMONTH, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.month = vu32;
	if(usbupc_query_device(dev, BRDYEAR, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.year = vu32;
	if(usbupc_query_device(dev, FRMMAJOR, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.firmmajor = vu32;
	if(usbupc_query_device(dev, FRMMINOR, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.firmminor = vu32;
	if(usbupc_query_device(dev, FRMBUILD, 0, &vu32) != 0)
		return -EIO;
	dev->eeinfo.firmbuild = vu32;
	usbupc_eeinfo_dump(&dev->eeinfo);

	for(i=0; i<36; i++) {
		usbupc_query_device(dev, USB_QRY_EEREAD, 200 + i * 2, &vu32);
		vu16 = vu32;
		usbupc_query_device(dev, USB_QRY_EEREAD, 201 + i * 2, &vu32);
		vu16 |= vu32<<8;
		dev->eeinfo.tmp_offset[i] = vu16;
	}
	for(i=0; i<32; i++) {
		usbupc_query_device(dev, USB_QRY_EEREAD, 300+i, &vu32);
		dev->eeinfo.tmp_gain[i] = vu32;
	}
	return EOK;
}

int usbupc_init_device( pUSBuPcDevice_t dev )
{
	int i;
	dev->adc.dblbuffer.data = kmalloc(DOUBLE_BUFFER_SIZE, GFP_KERNEL);
	if(dev->adc.dblbuffer.data == NULL)
		dev->adc.dblbuffer.size=0;
	else
		dev->adc.dblbuffer.size = DOUBLE_BUFFER_SIZE;
	usbupc_buf_init(&dev->adc.buffer, ADC_BUFFER_SIZE);
	for(i=0; i<ISO_NUM_URB; i++)
		dev->adc.isourb[i].pbuffer = &dev->adc.buffer;
	dev->iso_context.busy = 0;
	dev->usbint.pending = 0;
	return EOK;
}

void usbupc_uninit_device( pUSBuPcDevice_t dev )
{
	usbupc_isoxfr_stop(dev);
	usbupc_int_stop(dev);
	usbupc_buf_uninit(&dev->adc.buffer);
	if(dev->adc.dblbuffer.size > 0) kfree(dev->adc.dblbuffer.data);
}

/**
 * End of File
 */
