/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * ioctl.c -- I/O Control Source File
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "version.h"
#include "usbupc.h"
#include "usbio.h"
#include "buffer.h"
#include "ioctl.h"

int usbupc_ioctl_query( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[2];
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));

	switch(param[0]) {
	case BRDSERIALNO:
		param[0] = dev->eeinfo.serialnumber;
		break;
	case BRDREV:
		param[0] = dev->eeinfo.revision;
		break;
	case BRDTYPE:
		param[0] = dev->eeinfo.boardtype;
		break;
	case BRDYEAR:
		param[0] = dev->eeinfo.year;
		break;
	case BRDMONTH:
		param[0] = dev->eeinfo.month;
		break;
	case BRDDAY:
		param[0] = dev->eeinfo.day;
		break;
	case BRDBUSTYPE:
		param[0] = PC_BUS_USB;
		break;
	case DRVMAJOR:
		param[0] = (USBUPCDRV_VERSION>>16)&0xff;
		break;
	case DRVMINOR:
		param[0] = (USBUPCDRV_VERSION>>8)&0xff;
		break;
	case DRVBUILD:
		param[0] = USBUPCDRV_VERSION&0xff;
		break;
	case FRMMAJOR:
		param[0] = dev->eeinfo.firmmajor;
		break;
	case FRMMINOR:
		param[0] = dev->eeinfo.firmminor;
		break;
	case FRMBUILD:
		param[0] = dev->eeinfo.firmbuild;
		break;
	case ADNUMCHAN:
		param[0] = dev->eeinfo.adcchan;
		break;
	case ADMAXFREQ:
		param[0] = dev->eeinfo.adcfreq;
		break;
	case ADNUMTMP:
		param[0] = dev->eeinfo.tmpchan;
		break;
	case ADAMBCHAN:
		if(param[1] > dev->eeinfo.ambchan)
			return -EINVAL;
		param[0] = dev->eeinfo.tmp_ambch[param[1]];
		break;
	case ADBUSY:
		param[0] = dev->adc.status;
		break;
	case ADUNREAD:
		param[0] = usbupc_buf_getunread(&dev->adc.buffer);
		break;
	case ADBUFFSIZE:
		param[0] = dev->adc.buffer.size;
		break;
	case ADFIFOSIZE:
		param[0] = 512;
		break;
	case ADFIFOOVER:
	case ADBUFFOVER:
		param[0] = dev->adc.buffer.overrun;
		break;
	case ADBUFFALLOC:
		if(dev->adc.buffer.size > 0)
			param[0] = 1;
		else
			param[0] = 0;
		break;
	case ADNUMRANGES:
		param[0] = dev->eeinfo.adc_numranges;
		break;
	case ADRANGES:
		if(param[1] >= dev->eeinfo.adc_numranges)
			return -EINVAL;
		param[0] = dev->eeinfo.adc_ranges[param[1]];
		break;
	case ADNUMMODES:
		param[0] = dev->eeinfo.adc_nummodes;
		break;
	case ADMODES:
		if(param[1] >= dev->eeinfo.adc_nummodes)
			return -EINVAL;
		param[0] = dev->eeinfo.adc_modes[param[1]];
		break;
	case ADNUMTRIG:
		param[0] = dev->eeinfo.adc_numtrig;
		break;
	case ADTRIG:
		if(param[1] >= dev->eeinfo.adc_numtrig)
			return -EINVAL;
		param[0] = dev->eeinfo.adc_trig[param[1]];
		break;
	case ADNUMTRIGMODE:
		param[0] = dev->eeinfo.adc_numtrigmode;
		break;
	case ADTRIGMODE:
		if(param[1] >= dev->eeinfo.adc_numtrigmode)
			return -EINVAL;
		param[0] = dev->eeinfo.adc_trig[param[1]];
		break;
	case ADNUMCLK:
		param[0] = dev->eeinfo.adc_numclk;
		break;
	case ADCLK:
		if(param[1] >= dev->eeinfo.adc_numclk)
			return -EINVAL;
		param[0] = dev->eeinfo.adc_clk[param[1]];
		break;
	case ADSETDATAMODE:
		dev->adc.datamode = param[1] & 0x1l;
		if(param[1] > 1)
			return -EINVAL;
		break;
	case DANUMCHAN:
		param[0] = dev->eeinfo.dacchan;
		break;
	case CTNUM:
		param[0] = dev->eeinfo.cntchan;
		break;
	case DIONUMPORT:
		param[0] = dev->eeinfo.diochan;
		break;
	default:
		param[0] = 0;
		param[1] = 0;
	}
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}

int usbupc_dio_write( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[2], port, value;
	unsigned char buf[64];
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	value = param[1];
	if(port >= dev->eeinfo.diochan)
		return -EINVAL;
	port = dev->eeinfo.dio_route[port];
	if((dev->eeinfo.cntchan > 0) && (port >= (dev->eeinfo.diochan - 3)))
		port = 27 + (port - (dev->eeinfo.diochan - 3));
	buf[0] = USB_DIOWRITE>>8;
	buf[1] = USB_DIOWRITE;
	buf[2] = port>>8;
	buf[3] = port;
	buf[4] = value>>8;
	buf[5] = value;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	return EDRE_OK;
}

int usbupc_dio_read( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[1],port;
	unsigned char buf[64];
	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port = param[0];
	if(port >= dev->eeinfo.diochan)
		return -EINVAL;
	port = dev->eeinfo.dio_route[port];		
	if((dev->eeinfo.cntchan > 0) && (port >= (dev->eeinfo.diochan - 3)))
		port = 27 + (port - (dev->eeinfo.diochan - 3));
	buf[0] = USB_DIOREAD>>8;
	buf[1] = USB_DIOREAD;
	buf[2] = port>>8;
	buf[3] = port;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	param[0] = buf[2]<<8 | buf[3];
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}

int usbupc_cnt_write( pUSBuPcDevice_t dev,unsigned long arg )
{
	unsigned long param[2], port, value;
	unsigned char buf[64];
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	value = param[1];
	if(port >= dev->eeinfo.cntchan) return -EIO;
	buf[0] = USB_CTWRITE>>8;
	buf[1] = USB_CTWRITE;
	buf[2] = port>>8;
	buf[3] = port;
	buf[4] = value>>8;
	buf[5] = value;
	if(usbupc_bulkpacket(dev, buf, 64)!=EOK)
		return -EIO;
	if(buf[1] != 0)
		return -ERR;
	return EDRE_OK;
}

int usbupc_cnt_read( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[1], port;
	unsigned char buf[64];
	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	port = param[0];
	if(port >= dev->eeinfo.cntchan)
		return -EIO;
	buf[0] = USB_CTREAD>>8;
	buf[1] = USB_CTREAD;
	buf[2] = port>>8;
	buf[3] = port;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	param[0] = buf[2]<<8 | buf[3];
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}

int usbupc_cnt_config( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[5], port, mode, bcd, clksrc, gatesrc;
	unsigned char buf[64];
	copy_from_user(param, (void *)arg, 5*sizeof(unsigned long));
	port = param[0];
	mode = param[1];
	bcd = param[2];
	clksrc = param[3];
	gatesrc = param[4];
	if(port >= dev->eeinfo.cntchan)
		return -EIO;
	buf[0] = USB_CTCONFIG>>8;
	buf[1] = USB_CTCONFIG;
	buf[2] = port>>8;
	buf[3] = port;
	buf[4] = mode;
	buf[5] = bcd;
	buf[6] = clksrc;
	buf[7] = gatesrc;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	return EDRE_OK;
}

int usbupc_cnt_softgate( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[2], port, mode;
	unsigned char buf[64];
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	mode = param[1];
	if(port>= dev->eeinfo.cntchan)
		return -EIO;
	buf[0] = USB_CTSOFTGATE>>8;
	buf[1] = USB_CTSOFTGATE;
	buf[2] = port>>8;
	buf[3] = port;
	buf[4] = mode>>8;
	buf[5] = mode;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	return EDRE_OK;
}

int usbupc_int_config( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[3], src, mode, type;
	unsigned char buf[BULKDATAMAX];
	copy_from_user(param, (void *)arg, 3*sizeof(unsigned long));
	src = param[0];
	mode = param[1];
	type = param[2]; //ignored
	if(dev->eeinfo.irqchan == 0)
		return -EIO;
	if(src >= dev->eeinfo.irqchan)
		return -EINVAL;
	if(mode > 1)
		mode=1;
	dev->usbint.config &= 0xff - (1<<src);
	dev->usbint.config |= mode << src;
	buf[0] = USB_IRQCFG>>8;
	buf[1] = USB_IRQCFG;
	buf[2] = 0;
	buf[3] = dev->usbint.config;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	return EDRE_OK;
}

int usbupc_int_enable( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned char buf[BULKDATAMAX];
	buf[0] = USB_IRQEN>>8;
	buf[1] = USB_IRQEN;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	return EDRE_OK;
}

int usbupc_int_disable( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned char buf[BULKDATAMAX];
	buf[0] = USB_IRQDIS>>8;
	buf[1] = USB_IRQDIS;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	return EDRE_OK;
}

int usbupc_int_wait( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[1];
	spin_lock(&dev->usbint.lock);
	if(dev->usbint.pending > 0) {
		spin_unlock(&dev->usbint.lock);
		return -EBUSY;
	}
	dev->usbint.pending = 1;
	spin_unlock(&dev->usbint.lock);
	init_waitqueue_head(&dev->usbint.wait);
	usbupc_irq_queue(dev);
	//  interruptible_sleep_on(&dev->usbint.wait); /*** hang .... ***/
	spin_lock(&dev->usbint.lock);
	usb_unlink_urb(dev->usbint.urb);
	usb_free_urb(dev->usbint.urb);
	param[0] = dev->usbint.source;
	dev->usbint.pending = 0;
	spin_unlock(&dev->usbint.lock);
	copy_to_user((void *)arg, param, sizeof(unsigned long));
	return EDRE_OK;
}

int usbupc_int_release( pUSBuPcDevice_t dev, unsigned long arg )
{
	usbupc_int_stop(dev);
	return EDRE_OK;
}

int usbupc_dac_write( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[2], port;
	long value;
	unsigned short bin;
	unsigned char buf[BULKDATAMAX];
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	port = param[0];
	value = (long)param[1];
	if(port >= dev->eeinfo.dacchan)
		return -EINVAL;
	if((value > 10000000) || (value < -10000000))
		return -EINVAL;
	bin = usbupc_dac_volttobin(port, value);
	buf[0] = USB_DACWRITE>>8;
	buf[1] = USB_DACWRITE;
	buf[2] = bin>>8;
	buf[3] = bin;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != 0)
		return -EIO;
	return EDRE_OK;
}

int usbupc_adc_single( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[2], gain, chan;
	long uv, denom;
	int er;
	unsigned short bin;
	spin_lock(&dev->adc.lock);
	if(dev->adc.status)
		goto usbupc_adcsingle_busy;
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	chan = param[0];
	gain = param[1];
	if(chan >= dev->eeinfo.adcchan || gain > 9 || dev->adc.range > 3)
		goto usbupc_adcsingle_einval;
	if(chan >= (dev->eeinfo.adcchan - dev->eeinfo.tmpchan)) {
		/* temperature device */
		chan -= dev->eeinfo.adcchan - dev->eeinfo.tmpchan;
		er = usbupc_tmp_read(dev, chan, gain, dev->adc.range, &bin);
		if(er == -25 || er == -35)
			goto usbupc_adcsingle_eio;
		uv = usbupc_tmp_bintovolt(bin);
		if(chan >= ((dev->eeinfo.tmpchan / 8) * 8)) {
			uv += dev->eeinfo.tmp_offset[(chan % 8) + 32] - 32000;
		}
		else {
			denom = dev->eeinfo.tmp_gain[chan];
			uv += dev->eeinfo.tmp_offset[chan] - 32000;
			uv *= 8;
			uv /= denom;
		}
	}
	else {
		/* A/D device */
		er = usbupc_adc_read(dev, chan, gain, dev->adc.range, &bin);
		if(er == -25 || er == -35)
			goto usbupc_adcsingle_eio;
		uv = usbupc_adc_bintovolt(bin, dev->adc.range, gain);
	}
	copy_to_user((void *)arg, &uv, sizeof(long));
	spin_unlock(&dev->adc.lock);
	return EDRE_OK;

usbupc_adcsingle_eio:
	spin_unlock(&dev->adc.lock);
	return -EIO;
usbupc_adcsingle_busy:
	spin_unlock(&dev->adc.lock);
	return -EBUSY;
usbupc_adcsingle_einval:
	spin_unlock(&dev->adc.lock);
	return -EINVAL;
}

int usbupc_adc_config( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[2];
	spin_lock(&dev->adc.lock);
	if(dev->adc.status)
		goto usbupc_adc_config_busy;
	copy_from_user(param, (void *)arg, 2*sizeof(unsigned long));
	if(param[0] > 3)
		param[0]=3;
	dev->adc.range = param[0];
	spin_unlock(&dev->adc.lock);
	return EDRE_OK;

usbupc_adc_config_busy:
	spin_unlock(&dev->adc.lock);
	return -EBUSY;
}

int usbupc_adc_stop( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned char buf[BULKDATAMAX];
	spin_lock(&dev->adc.lock);
	usbupc_isoxfr_stop(dev);
	buf[0] = USB_ADCSTOP>>8;
	buf[1] = USB_ADCSTOP;
	dev->adc.status = IDLE;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		goto usbupc_adc_stop_eio;
	spin_unlock(&dev->adc.lock);
	return EDRE_OK;

usbupc_adc_stop_eio:
	spin_unlock(&dev->adc.lock);
	return -EIO;
}

int usbupc_adc_start( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned char buf[BULKDATAMAX];
	spin_lock(&dev->adc.lock);

	if(dev->iso_ep != EMPTY_ENDPOINT) {
		if(usbupc_isoxfr_start(dev)) {
			err("Failed to start isochronous transfers.");
			return -ENOMEM;
		}
	}
	buf[0] = USB_ADCSTART>>8;
	buf[1] = USB_ADCSTART;
	if(dev->adc.status)
		goto usbupc_adc_start_busy;
	if(usbupc_bulkpacket(dev, buf, 64))
		goto usbupc_adc_start_bulkerr;
	dev->adc.status = BUSY;
	spin_unlock(&dev->adc.lock);
	return EDRE_OK;

usbupc_adc_start_bulkerr:
	err("bulk error");
	usbupc_isoxfr_stop(dev);
	spin_unlock(&dev->adc.lock);
	return -EIO;
usbupc_adc_start_busy:
	err("busy");
	spin_unlock(&dev->adc.lock);
	return -EBUSY;
}

int usbupc_adc_clksrc( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned char buf[BULKDATAMAX];
	unsigned long param[1], clksrc, trigsrc;
	spin_lock(&dev->adc.lock);
	if(dev->adc.status)
		goto usbupc_adc_clksrc_busy;
	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	clksrc = param[0] & 0xff;
	trigsrc = (param[0]>>8) & 0xff;
	if(clksrc > 1) clksrc = 1;
	if(trigsrc>1) trigsrc = 1;
	buf[0] = USB_ADCLKSRC>>8;
	buf[1] = USB_ADCLKSRC;
	buf[2] = clksrc>>8;
	buf[3] = clksrc;
	buf[4] = trigsrc>>8;
	buf[5] = trigsrc;
	if(usbupc_bulkpacket(dev, buf, 64))
		goto usbupc_adc_clksrc_errio;
	if(buf[1] != EDRE_OK)
		goto usbupc_adc_clksrc_errio;
	spin_unlock(&dev->adc.lock);
	return EDRE_OK;

usbupc_adc_clksrc_errio:
	spin_unlock(&dev->adc.lock);
	return -EIO;
usbupc_adc_clksrc_busy:
	spin_unlock(&dev->adc.lock);
	return -EBUSY;
}

int usbupc_adc_freq( pUSBuPcDevice_t dev, unsigned long arg )
/**
 * TODO: add oversamling functionality.
 */
{
	unsigned long param[1], freq, val;
	spin_lock(&dev->adc.lock);
	if(dev->adc.status)
		goto usbupc_adc_freq_busy;
	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	freq = param[0];
	if(freq > 250000 || freq == 0)
		goto usbupc_adc_freq_einval;
	//  if(freq<2000) freq=usbupc_adc_getoversamplefreq(&dev->adc,freq);
	//  else dev->adc.oversample=0;
	val = 10000000 / freq;
	if(usbupc_adc_setfrequency(dev, val) != EOK)
		goto usbupc_adc_freq_eio;
	freq = 10000000 / val;
	//  if(dev->adc.oversample>0) freq/=dev->adc.oversample;
	dev->adc.frequency = freq;
	copy_to_user((void *)arg, &freq, sizeof(unsigned long));
	spin_unlock(&dev->adc.lock);
	return EDRE_OK;

usbupc_adc_freq_busy:
	spin_unlock(&dev->adc.lock);
	return -EBUSY;
usbupc_adc_freq_einval:
	spin_unlock(&dev->adc.lock);
	return -EINVAL;
usbupc_adc_freq_eio:
	spin_unlock(&dev->adc.lock);
	return -EIO;
}

int usbupc_adc_chlst( pUSBuPcDevice_t dev, unsigned long arg )
{
	unsigned long param[33], ch;
	unsigned char chan, gain, range, chlstsz;
	if(dev->adc.status)
		goto usbupc_adc_chlst_busy;
	copy_from_user(param, (void *)arg, sizeof(unsigned long));
	chlstsz = param[0];
	if(chlstsz > dev->eeinfo.adcchan)
		goto usbupc_adc_chlst_einval;
	copy_from_user(param, (void*)arg, (chlstsz*2+1)*sizeof(unsigned long));
	usbupc_adc_reset(&dev->adc);
	usbupc_buf_reset(&dev->adc.buffer);
	if(usbupc_adc_chanlstrst(dev))
		goto usbupc_adc_chlst_eio;
	for(ch=0; ch<chlstsz; ch++) {
		chan = (unsigned char)param[ch * 2 + 1];
		gain = (unsigned char)(param[ch * 2 + 2] & 0x0f);
		range = (unsigned char)((param[ch * 2 + 2] >> 8) & 0x0f);
		dev->adc.chanlst[ch] = chan;
		dev->adc.gainlst[ch] = gain;
		dev->adc.rangelst[ch] = range;
		if(usbupc_adc_addchan(dev, chan, gain, range) != EOK)
			goto usbupc_adc_chlst_eio;
	}
	if(chlstsz > 1) {
		ch = 0;
		chan = (unsigned char)param[ch * 2 + 1];
		gain = (unsigned char)(param[ch * 2 + 2] & 0x0f);
		range = (unsigned char)((param[ch * 2 + 2] >> 8) & 0x0f);
		if(usbupc_adc_addchan(dev, chan, gain, range) != EOK)
			goto usbupc_adc_chlst_eio;
	}
	dev->adc.chanlstsz = chlstsz;
	usbupc_adc_chanlstdump(&dev->adc);
	return EDRE_OK;

usbupc_adc_chlst_busy:
	spin_unlock(&dev->adc.lock);
	return -EBUSY;
usbupc_adc_chlst_einval:
	spin_unlock(&dev->adc.lock);
	return -EINVAL;
usbupc_adc_chlst_eio:
	spin_unlock(&dev->adc.lock);
	return -EIO;
}

/* private helper functions */
int usbupc_adc_chanlstrst( pUSBuPcDevice_t dev )
{
	unsigned char buf[BULKDATAMAX];
	buf[0] = USB_ADCCHLSRST>>8;
	buf[1] = USB_ADCCHLSRST;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != EDRE_OK)
		return -EIO;
	return EDRE_OK;
}

unsigned short usbupc_dac_volttobin( int channel, long value )
{
	unsigned short bin;
	bin = (value / 1221) + 8192;
	bin |= channel<<14;
	return bin;
}

long usbupc_tmp_bintovolt( unsigned short bin )
{
	long uv;
	bin &= 0x3fff;
	if(bin < 8192) uv = (bin * 1526) / 5;
	else
	{
		bin &= 8191;
		bin = 8191 - bin;
		uv = ((bin * 1526) / 5);
		uv *= -1;
	}
	return uv;
}

unsigned long usbupc_adc_getoversamplefreq( pADC_t adc, unsigned long freq )
{
	unsigned long of, chl, cnt;
	cnt = adc->minfreq / freq;
	if(cnt < 1) cnt = 1;
	chl = adc->chanlstsz;
	adc->oversample = ((cnt / chl) + 1) * chl + 1;
	of = adc->oversample * freq;
	return of;
}

int usbupc_adc_setfrequency( pUSBuPcDevice_t dev, unsigned short value )
{
	unsigned char buf[BULKDATAMAX];
	unsigned short val;
	val = value - 1;
	buf[0] = USB_ADCFREQ>>8;
	buf[1] = USB_ADCFREQ;
	buf[2] = val>>24;
	buf[3] = val>>16;
	buf[4] = val>>8;
	buf[5] = val;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != EDRE_OK)
		return -EIO;
	return EDRE_OK;
}

void usbupc_adc_reset(pADC_t adc)
{
	spin_lock(&adc->buffer.lock);
	adc->chanlstsz = 0;
	adc->chanidx = 0;
	spin_unlock(&adc->buffer.lock);
}

int usbupc_adc_addchan( pUSBuPcDevice_t dev, unsigned char channel,
			unsigned char gain, unsigned char range )
{
	unsigned char buf[BULKDATAMAX];
	unsigned char chmx, gnmx;
	usbupc_adc_encodechan(channel, gain, range, &chmx, &gnmx);
	buf[0] = USB_ADCADDCH>>8;
	buf[1] = USB_ADCADDCH;
	buf[2] = chmx;
	buf[3] = gnmx;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != EDRE_OK)
		return -ERR;
	return EDRE_OK;
}

void usbupc_adc_encodechan( unsigned char channel, unsigned char gain,
			    unsigned char range, unsigned char *chanmux,
			    unsigned char *gainmux )
{
	const unsigned char
	SEMUX[17] = {	0x14,0x15,0x16,0x17,0x18,0x19,0x1a,0x1b,0x24,
			0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x30 };
	const unsigned char DIFFMUX[8] = { 0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b };
	const unsigned char RANGEBIP[9] = { 	0x04,0x00,0x08,0x05,0x01,
						0x09,0x06,0x02,0x0a};
	const unsigned char RANGEUNI[9] = {	0x010,0x10,0x18,0x15,0x11,
						0x19,0x16,0x12,0x1a };
	if(range < 2)
		*chanmux = SEMUX[channel];
	else
		*chanmux = DIFFMUX[channel];

	if((range == 0) || (range == 2))
		 *gainmux=RANGEUNI[gain];
	else
		*gainmux=RANGEBIP[gain];
}

int usbupc_adc_read( pUSBuPcDevice_t dev, unsigned char channel,
		     unsigned char gain, unsigned char range, unsigned short *bin)
{
	unsigned char buf[BULKDATAMAX];
	unsigned char chmx, gnmx;
	usbupc_adc_encodechan(channel, gain, range, &chmx, &gnmx);
	buf[0] = USB_ADCREAD>>8;
	buf[1] = USB_ADCREAD;
	buf[2] = chmx;
	buf[3] = gnmx;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != EDRE_OK)
		return -EIO;
	*bin = (buf[2]<<8) | buf[3];
	return EOK;
}

int usbupc_tmp_read( pUSBuPcDevice_t dev, unsigned char channel,
		     unsigned char gain, unsigned char range, unsigned short *bin )
{
	unsigned char buf[BULKDATAMAX];
	const unsigned char
	MUX[18] = { 	0x04,0x05,0x06,0x07,0x14,0x15,0x16,0x17,
			0x08,0x09,0x0a,0x0b,0x18,0x19,0x1a,0x1b,0x20,0x30};
	unsigned char board = 2;
	if(dev->eeinfo.tmpchan > 18)
	{
		if((channel >= 16) && (channel <= 33)) {
			if(channel < 32)
				board = 3;
			channel -= 16;
		}
		else if(channel >= 34) {
			channel -= 18;
			board = 3;
		}
	}
	if((dev->eeinfo.tmpchan == 9) && (channel == 8)) channel = 16;
	buf[0] = board;
	buf[1] = USB_ADCREAD;
	buf[2] = MUX[channel];
	buf[3] = 0;
	if(usbupc_bulkpacket(dev, buf, 64) != EOK)
		return -EIO;
	if(buf[1] != EDRE_OK)
		return -EIO;
	*bin = (buf[2]<<8) | buf[3];
	return EOK;
}

void usbupc_adc_chanlstdump(pADC_t adc)
{
	int i;
	dbg("usbupc_adc_chanlstdump: start(%d)", adc->chanlstsz);
	for(i=0; i<adc->chanlstsz; i++) {
		dbg( "usbupc_adc_chanlstdump: chan: %d, gain: %d, range: %d",
			adc->chanlst[i], adc->gainlst[i], adc->rangelst[i]);
	}
	dbg("usbupc_adc_chanlstdump: end");
}

void usbupc_int_stop(pUSBuPcDevice_t dev)
{
	spin_lock(&dev->usbint.lock);
	if(dev->usbint.pending > 0) {
		wake_up_interruptible(&dev->usbint.wait);
		usb_unlink_urb(dev->usbint.urb);
	}
	spin_unlock(&dev->usbint.lock);
}

/**
 * End of File
 */
