/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * buffer.c -- Buffer manager source file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#include "stdinc.h"
#include "usbupc.h"
#include "buffer.h"

int usbupc_buf_init( pADCBuffer_t buf, unsigned long length )
{
	spin_lock_init(&buf->lock);
	buf->data = (unsigned short *)vmalloc(length * sizeof(unsigned short));
	if(buf->data == NULL)
		buf->size = 0;
	else
		buf->size = length;
	usbupc_buf_reset(buf);
	return EDRE_OK;
}

void usbupc_buf_uninit( pADCBuffer_t buf )
{
	if(buf->size > 0)
		vfree(buf->data);
}

void usbupc_buf_reset( pADCBuffer_t buf )
{
	spin_lock(&buf->lock);
	buf->samples = 0;
	buf->head = 0;
	buf->tail = 0;
	buf->wrap = 0;
	buf->overrun = 0;
	spin_unlock(&buf->lock);
}

int usbupc_buf_adddata( pADCBuffer_t buf, void *buffer, unsigned short bytes )
{
	int eerr = EDRE_OK;
	unsigned long samples;
	spin_lock(&buf->lock);
	samples = bytes / 2;
	if(samples > (buf->size - buf->samples)) {
		samples = buf->size - buf->samples;
		eerr = EDRE_FAIL;
		buf->overrun = 1;
	}
	if(samples > ((buf->size - 1) - buf->head)) {
		memcpy(	&buf->data[buf->head], buffer,
			((buf->size - 1) - buf->head)*sizeof( unsigned short));
		samples -= ((buf->size - 1) - buf->head);
		buf->samples += ((buf->size - 1) - buf->head);
		buf->head = 0;
		buf->wrap++;
	}
	memcpy(&buf->data[buf->head], buffer, samples*sizeof(unsigned short));
	buf->head += samples;
	buf->samples += samples;
	spin_unlock(&buf->lock);
	return eerr;
}

unsigned long usbupc_buf_getdataraw( pADCBuffer_t buf, unsigned short *buffer,
					unsigned long samples )
{
	unsigned long copied;
	spin_lock(&buf->lock);
	if(samples > buf->samples)
		samples = buf->samples;
	copied = samples;
	if(samples > ((buf->size - 1) - buf->tail)) {
		memcpy( buffer, &buf->data[buf->tail],
			((buf->size - 1) - buf->tail) * sizeof(unsigned short));
		samples -= ((buf->size - 1) - buf->tail);
		buf->samples += ((buf->size - 1) - buf->tail);
		buf->tail = 0;
	}
//	memcpy(buffer, &buf->data[buf->tail], samples*sizeof(unsigned short));
	buf->tail += samples;
	buf->samples -= samples;
	spin_unlock(&buf->lock);
	return copied;
}

unsigned long usbupc_buf_getdata( pADC_t adc, long *buffer,
				  unsigned long samples )
{
	pADCBuffer_t buf = &adc->buffer;
	unsigned long i;
	int range, gain;
//	buf->samples -= samples;
//	return samples;
	spin_lock(&buf->lock);
	if(samples > buf->samples)
		samples = buf->samples;
	for(i=0; i<samples; i++) {
		range = adc->rangelst[adc->chanidx];
		gain = adc->gainlst[adc->chanidx];
//		buffer[i] = usbupc_adc_bintovolt(buf->data[buf->tail], range, gain);
		if(++adc->chanidx == adc->chanlstsz)
			adc->chanidx = 0;
		if(++buf->tail == buf->size)
			buf->tail = 0;
	}
	buf->samples -= samples;
	spin_unlock(&buf->lock);
	return samples;
}

unsigned long usbupc_buf_getunread( pADCBuffer_t buf )
{
	unsigned long samples;
	spin_lock(&buf->lock);
	samples = buf->samples;
	spin_unlock(&buf->lock);
	return samples;
}


long usbupc_adc_bintovolt( unsigned short bin, unsigned char channel, 
													unsigned char gain, unsigned char offset, 
													unsigned char gain_error )
{
	unsigned char GAIN[8] = {1, 2, 4, 5, 8, 10, 16, 20};
	long uv;
	
	bin &= 0x0fff;
	
	if(channel > 0x07) {
		uv = (bin * 9765) - 9999360;
	}
	else {
	
		if(bin > 0x07ff) {
			bin = 0x1000 - bin;
			uv = (bin * 9765) / GAIN[gain];
			uv = -uv;
		}
		else {
			bin &= 0x07ff;
			uv = (bin * 9765) / GAIN[gain];
		}
		
	}
	
	uv += (long)( ((80 - offset) * 9765) / GAIN[7]);
	uv = (uv * 1000) / (900 + gain_error);
	
	return uv;
}

/**
 * End of File
 */
