/*******************************************************************************
 *                 Eagle Technology - Cape Town, South Africa
 *             Copyright (C) 2004 - 2005 - All Rights Reserved!
 *                 www.eagledaq.com - eagle@eagle.co.za
 *******************************************************************************
 *
 * usbupc.h -- USBuPc main header file
 *
 *******************************************************************************
 *                              LICENSE AGREEMENT
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ******************************************************************************/
#ifndef _USBUPC_H__
#define _USBUPC_H__

/*General Constans*/
#define BULKDATAMAX 64                  /**/
#define PACKET_DIR_OUT 1
#define PACKET_DIR_IN 0
#define URB_TIMEOUT 100
#define STOPPED 0
#define STARTED 1
#define MAX_USBUPC_DEVICES 256
#define EOK 0
#define ERR -1
#define EMPTY_ENDPOINT 0xffff
#define BUSY 1
#define IDLE 0
#define ISO_NUM_URB 2
#define ISO_PIPESIZE 5120
#define DOUBLE_BUFFER_SIZE 8192
#define ADC_BUFFER_SIZE 1048576

/*macros*/
#define TYPE(dev)   (MINOR(dev) >> 4)  /* high nibble */
#define NUM(dev)    (MINOR(dev) & 0xf) /* low  nibble */
#define dbg2(format, arg...) printk(KERN_WARNING __FILE__ ": " format "\n" , ## arg)

struct EepromInfo_t {
	unsigned long serialnumber;
	unsigned short month, day, year;
	unsigned short revision;
	unsigned short boardtype;
	unsigned short firmmajor, firmminor, firmbuild;
	unsigned char diochan, adcchan, dacchan, cntchan;
	unsigned long adc_freq;
	unsigned long dac_freq;
	unsigned char dac_speed;
	unsigned char dac_power;
	unsigned char adc_offset[12];
	unsigned char adc_gain_error;
	unsigned char dac_offset[2];
	unsigned char dac_gain_error[2];
	unsigned short adc_numranges;
	unsigned short adc_ranges[20];
	unsigned short adc_nummodes;
	unsigned short adc_modes[20];
	unsigned short adc_numtrig;
	unsigned short adc_trig[20];
	unsigned short adc_numtrigmode;
	unsigned short adc_trigmode[10];
	unsigned short adc_numclk;
	unsigned short adc_clk[10];
};
#define pEepromInfo_t struct EepromInfo_t*

struct ADCBuffer_t {
	unsigned short *data;
	unsigned long size;
	unsigned long samples;
	unsigned long head;
	unsigned long tail;
	unsigned long wrap;
	unsigned char overrun;
	spinlock_t lock;
};
#define pADCBuffer_t struct ADCBuffer_t*

struct DblBuffer_t {
	void *data;
	unsigned short size;
};
#define pDblBuffer_t struct DblBuffer_t*

struct IsoContext_t {
	spinlock_t lock;
	unsigned char busy;
	unsigned char pending;
	struct completion done;
	unsigned long errors;
};
#define pIsoContext_t struct IsoContext_t*

struct IsoUrb_t {
	int index;
	int pipesize;
	int allocated;
	int interval;
	unsigned int endpoint;
	pADCBuffer_t pbuffer;
	pIsoContext_t context;
	struct urb *urb;
};
#define pIsoUrb_t struct IsoUrb_t*

struct ADC_t {
	spinlock_t lock;
	unsigned char status;
	unsigned char range;
	unsigned char chanlst[8];
	unsigned char gainlst[8];
	unsigned char rangelst[8];
	unsigned char chanlstsz;
	unsigned long frequency;
	unsigned long minfreq;
	unsigned long oversample;
	unsigned char chanidx;
	unsigned char datamode;
	struct ADCBuffer_t buffer;
	struct DblBuffer_t dblbuffer;
	struct IsoUrb_t isourb[ISO_NUM_URB];
};
#define pADC_t struct ADC_t*

struct USBuPcDevice_t {
	struct usb_device *usbdev;
	struct usb_interface *usbif;
	wait_queue_head_t wait;
	wait_queue_head_t remove_ok;
	spinlock_t lock;
	spinlock_t spl_bulk;
	int state;
	int got_mem;
	int total_buffer_size;
	unsigned int bulk_ep;
	unsigned int iso_ep;
	int iso_ep_size;
	int iso_interval;
	struct IsoContext_t iso_context;
	int readptr;
	int minor;
	struct ADC_t adc;
	struct EepromInfo_t eeinfo;
	struct kref kref;
};
#define pUSBuPcDevice_t struct USBuPcDevice_t*

struct USBuPc_t {
	pUSBuPcDevice_t device[MAX_USBUPC_DEVICES];
	unsigned char allocated[MAX_USBUPC_DEVICES];
	int major;
	struct class_simple *device_class;
};

#endif /* _USBUPC_H_ */

/**
 * End of File
 */
